# encoding: utf-8
# license: gpl2p 

### BEGIN LICENSE NOTICE
# This file is part of the interconnect asset ontology framework (IO)
# Copyright (C) 2010 - 2013
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
### END LICENSE NOTICE

### BEGIN AUTHOR LIST
#
### END AUTHOR LIST

module Ghun
  module Base
    module Logging
      def init_logging(log_source=Log::Source::BASE,log_type=Log::Type::BASE,fallback_log_level=Log::Level::WARNING)
        @__log_source=log_source
        @__log_type=log_type
        @__fallback_log_level=fallback_log_level
      end
      private :init_logging

      def debug(log_type=@__log_type,*messages)
        log(Ghun::Log::Level::DEBUG,@__log_source,log_type,messages)
      end
      private :debug

      def trace(log_type=@__log_type,*messages)
        log(Ghun::Log::Level::TRACE,@__log_source,log_type,messages)
      end
      private :trace

      def info(log_type=@__log_type,*messages)
        log(Ghun::Log::Level::INFO,@__log_source,log_type,messages)
      end
      private :info

      def warn(log_type=@__log_type,*messages)
        log(Ghun::Log::Level::WARNING,@__log_source,log_type,messages)
      end
      private :warn

      def error(log_type=@__log_type,*messages)
        log(Ghun::Log::Level::ERROR,@__log_source,log_type,messages)
      end
      private :error

      def fatal(log_type=@__log_type,*messages)
        log(Ghun::Log::Level::FATAL,@__log_source,log_type,messages)
      end
      private :fatal

      def logger()
        Ghun::Base::Blackboard.logger
      end

      def log(log_level, log_source, log_type,messages)
        if logger.nil?() then
          if log_type.is_a?(Fixnum) then
            fallback_log(log_level,log_source,log_type,messages)
          else
            fallback_log(log_level,log_source,@__log_type,[log_type].concat(messages))
          end
        else
          if log_type.is_a?(Fixnum) then
            logger.log(log_level,log_source,log_type,messages)
          else
            logger.log(log_level,log_source,@__log_type,[log_type].concat(messages))
          end
        end
      end
      private :log

      def fallback_log(log_level, log_source, log_type,messages)
        @__fallback_log_level=Ghun::Log::Level::DEBUG if @__fallback_log_level.nil?
        return unless @__fallback_log_level >= log_level
        if messages.is_a? Enumerable then
          messages.each do |message|
            fallback_log(log_level,log_source,log_type,message)
          end
        else
          if messages.is_a? Exception then
            fallback_log log_level,log_source,log_type,"Exception: #{messages.class}"
            fallback_log log_level,log_source,log_type,messages.message
            fallback_log log_level,log_source,log_type,messages.backtrace
          elsif messages.is_a? String then
            $stdout.puts build_message(log_level,log_source,log_type,messages)
          elsif messages.respond_to? :to_s then
            fallback_log(log_level,log_source,log_type,"Class: #{messages.class}; Content: #{messages.to_s}")
          else
            fallback_log(log_level,log_source,log_type,"Class: #{messages.class}; content not accessible")
          end
        end
      end
      private :fallback_log

      def build_message(log_level,log_source,log_type,message)
        output = ""
        output += "[#{Time.now.strftime('%a %b %d %H:%M:%S %Z %Y')}] "
        output += "[#{Ghun::Log::Level.output(log_level)}] "
        output += "[#{Ghun::Log::Source.output(log_source)}] "
        output += "[#{Ghun::Log::Type.output(log_type)}] "
        output += message
      end
      private :build_message
    end

    module Configuration
      def init_configuration
        return true
      end
      private :init_configuration

      def config
        Ghun::Base::Blackboard.config
      end

      def _reload_configuration
        self.reload_configuration if self.respond_to?(:reload_configuration, true)
      end

      def register
        config.register(self) unless config.nil?()
      end
      private :register

      def unregister
        config.unregister(self) unless config.nil?()
      end
      private :unregister
    end
  end
end
