# encoding: utf-8
# license: gpl2p 

### BEGIN LICENSE NOTICE
# This file is part of the interconnect asset ontology framework (IO)
# Copyright (C) 2010 - 2013
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
### END LICENSE NOTICE

### BEGIN AUTHOR LIST
#
### END AUTHOR LIST
require 'fileutils'
module Ghun
  module Log
    class StorageFile < Storage
      Ghun::Log::Handler::STORAGE_SELECTOR.register_member(self,:file)

      #FIXME needs reload and destroy?
      def initialize(name)
        super(name)
        config.declare_dynamic_key("log.storage.#{@name}.logdir".intern,:string,File.join(Ghun::Base::Blackboard.log_path,Ghun::Base::Blackboard.component.internal_name))
        @logdir=config[:"log.storage.#{@name}.logdir"]
        FileUtils::mkdir_p(@logdir,{:mode => 0750}) unless Dir.exists?(@logdir)
        @logfiles=Hash.new()
      end

      def disable
        @enabled=false
        @logfiles.each do |n,l|
          begin
            l.close unless (l.nil?) || (l.closed?)
          rescue => e
            if Ghun::Base::Blackboard.logger
              Ghun::Base::Blackboard.logger.error(Ghun::Log::Source::FILE,Ghun::Log::Type::BASE,"Failed to close logfile for queue #{n}",e)
            else
              puts "Failed to close logfile for queue #{n}"
            end
          end
        end
      rescue => e
        if Ghun::Base::Blackboard.logger
          Ghun::Base::Blackboard.logger.error(Ghun::Log::Source::FILE,Ghun::Log::Type::BASE,"Failed to disable #{self.class}",e)
        else
          puts "Failed to disable #{self.class}"
        end
      end

      def get_log_file(name)
        if (!(@logfiles.include?(name.intern)) || (@logfiles[name.intern].nil?) || (@logfiles[name.intern].closed?)) then
          @logfiles[name.intern]=File.open(File.join(@logdir,name),"a")
          Ghun::Base::Blackboard.logger.info(Ghun::Log::Source::FILE,Ghun::Log::Type::BASE,"Opened log file #{File.join(@logdir,name)}") if Ghun::Base::Blackboard.logger
        end
        @logfiles[name.intern]
      rescue => e
        @logfiles[name.intern].close if @logfiles[name.intern]
        @enabled=false
        if Ghun::Base::Blackboard.logger
          Ghun::Base::Blackboard.logger.error(Ghun::Log::Source::FILE,Ghun::Log::Type::BASE,"Exception occured during file operation", "Disabling #{self.class}", e)
        else
          puts "Exception occured during file operation"
          puts "Disabling #{self.class}"
        end
      end
      private :get_log_file

      def build_message(loglevel,logsource,logtype,queue,message)
        output = ""
        output += "[#{Time.now.strftime '%a %b %d %H:%M:%S %Z %Y'}] "
        output += "[#{Level.output loglevel}] "
        output += "[#{Source.output logsource}] "
        output += "[#{Type.output logtype}] "
        output += message
      end
      private :build_message

      def log(loglevel,logsource,logtype,queue,message)
        return unless appropriate_storage?(loglevel,logsource,logtype)
        get_log_file(queue).puts(build_message(loglevel,logsource,logtype,queue,message))
      rescue => e
        disable
        if Ghun::Base::Blackboard.logger
          Ghun::Base::Blackboard.logger.error(Ghun::Log::Source::FILE,Ghun::Log::Type::BASE,"Failed to write message '#{message}' from queue #{queue} to logfile",e)
        else
          puts "Failed to write message '#{message}' from queue #{queue} to logfile"
        end
      end
    end
  end
end
