# encoding: utf-8
# license: gpl2p 

### BEGIN LICENSE NOTICE
# This file is part of the interconnect asset ontology framework (IO)
# Copyright (C) 2010 - 2013
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
### END LICENSE NOTICE

### BEGIN AUTHOR LIST
#
### END AUTHOR LIST
module Ghun
  module Log

    Message=Struct.new(:log_level,:log_source,:log_type,:message)

    module Level
      DEBUG=5
      TRACE=4
      INFO=3
      WARNING=2
      ERROR=1
      FATAL=0

      def self.output(level)
        case level
        when FATAL
          "FATAL"
        when ERROR
          "ERROR"
        when WARNING
          "WARNING"
        when INFO
          "INFO"
        when TRACE
          "TRACE"
        when DEBUG
          "DEBUG"
        else
          ""
        end
      end

      def self.is_log_level?(value)
        Ghun::Config::TypeChecker::is_uint_range?(value,Log::Level::FATAL,Log::Level::DEBUG)
      end

      def self.to_log_level(value)
        Ghun::Config::TypeChecker::to_uint_range(value,Log::Level::FATAL,Log::Level::DEBUG,true)
      end

      Ghun::Config::TypeChecker.register_type(:"log_level",self.method(:"is_log_level?"),self.method(:"to_log_level"))
    end


    module Source
      NONE=0
      ALL=0
      @@sources=Hash.new()
      @@counter=1
      @@none=NONE
      @@all=ALL

      def self.register_log_source(name)
        n=name.upcase.intern
        return if @@sources.include?(n)
        @@sources[n]=n.to_s
        self.const_set(n,@@counter)
        @@all+=@@counter
        self.send(:remove_const,:ALL)
        self.const_set(:ALL,@@all)
        @@counter*=2
      end

      def self.unregister_log_source(name)
        n=name.upcase.intern
        return unless @@sources.include?(n)
        @@sources.delete(n)
        @@all-=self.const_get(n)
        self.send(:remove_const,:ALL)
        self.const_set(:ALL,@@all)
        self.send(:remove_const,n)
      end

      self.register_log_source("BASE")
      self.register_log_source("CONFIG")
      self.register_log_source("FILE")
      self.register_log_source("LOG")

      def self.output(source)
        output=""
        self.constants.each do |c|
          next if c==:ALL
          next if c==:NONE
          output+="#{c.to_s} " if (source&(self.const_get(c)))!=0
        end unless self.constants.nil?()
        output.strip!
        return output
      end

      def self.is_log_source?(value)
        Ghun::Config::TypeChecker::is_uint_range?(value,@@none,@@all)
      end

      def self.to_log_source(value)
        Ghun::Config::TypeChecker::to_uint_range(value,@@none,@@all,false)
      end

      Ghun::Config::TypeChecker.register_type(:"log_source",self.method(:"is_log_source?"),self.method(:"to_log_source"))
    end


    module Type
      NONE=0
      ALL=0
      @@types=Hash.new()
      @@counter=1
      @@none=NONE
      @@all=ALL

      def self.register_log_type(name)
        n=name.upcase.intern
        return if @@types.include?(n)
        @@types[n]=n.to_s
        self.const_set(n,@@counter)
        @@all+=@@counter
        self.send(:remove_const,:ALL)
        self.const_set(:ALL,@@all)
        @@counter*=2
      end

      def self.unregister_log_type(name)
        n=name.upcase.intern
        return unless @@types.include?(n)
        @@types.delete(n)
        @@all-=self.const_get(n)
        self.send(:remove_const,:ALL)
        self.const_set(:ALL,@@all)
        self.send(:remove_const,n)
      end

      def self.output(type)
        output=""
        self.constants.each do |c|
          next if c==:ALL
          next if c==:NONE
          output+="#{c.to_s} " if (type&(self.const_get(c)))!=0
        end unless self.constants.nil?()
        output.strip!
        return output
      end

      self.register_log_type("BASE")
      self.register_log_type("READ")
      self.register_log_type("WRITE")
      self.register_log_type("REPORT")
      self.register_log_type("EXTERNAL_LIBS")

      def self.is_log_type?(value)
        Ghun::Config::TypeChecker::is_uint_range?(value,@@none,@@all)
      end

      def self.to_log_type(value)
        Ghun::Config::TypeChecker::to_uint_range(value,@@none,@@all,false)
      end

      Ghun::Config::TypeChecker.register_type(:"log_type",self.method(:"is_log_type?"),self.method(:"to_log_type"))
    end
  end
end
