# encoding: utf-8
# license: gpl2p 

### BEGIN LICENSE NOTICE
# This file is part of the interconnect asset ontology framework (IO)
# Copyright (C) 2010 - 2013
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
### END LICENSE NOTICE

### BEGIN AUTHOR LIST
#
### END AUTHOR LIST
module Ghun
  module Log
    class Compatible

      DEBUG = 0
      INFO = 1
      WARN = 2
      ERROR = 3
      FATAL = 4
      UNKNOWN = 5

      attr_accessor :log_source, :log_type

      def initialize(log_source=Ghun::Log::Source::BASE,log_type=Ghun::Log::Type::BASE)
        reset(log_source,log_type)
      end

      def reset(log_source=Ghun::Log::Source::BASE,log_type=Ghun::Log::Type::BASE)
        @log_source=log_source
        @log_type=log_type
      end

      def debug(message)
        log(DEBUG,message)
      end

      def info(message)
        log(INFO,message)
      end

      def warn(message)
        log(WARN,message)
      end
      alias :warning :warn

      def error(message)
        log(ERROR,message)
      end

      def fatal(message)
        log(FATAL,message)
      end

      def unknown(message)
        log(UNKNOWN,message)
      end

      def log(log_level,message)
        case log_level
        when DEBUG
          level=Ghun::Log::Level::DEBUG
        when INFO
          level=Ghun::Log::Level::INFO
        when WARN
          level=Ghun::Log::Level::WARNING
        when ERROR
          level=Ghun::Log::Level::ERROR
        when FATAL
          level=Ghun::Log::Level::FATAL
        else
          level=Ghun::Log::Level::FATAL
        end
        Ghun::Base::Blackboard.logger.log(level,@log_source,@log_type,message) if Ghun::Base::Blackboard.logger
      end

      def debug?
        true
      end

      def info?
        true
      end

      def warn?
        true
      end
      alias :warning? :warn?

      def error?
        true
      end

      def fatal?
        true
      end

      def add(level,_unused,_facility)
        message=yield
        log(level,message)
      end
    end
  end
end
