#!/bin/bash

%CREDENTIALS%

declare -i number_of_vms=%NUMBER_OF_VMS%
declare -i number_of_networks=%NUMBER_OF_NETWORKS%

declare -a vms
declare -a images
declare -a vm_to_network
declare -a networks

demo="%DEMONSTRATOR%"

%FILL_VMS%

%FILL_IMAGES%

%FILL_NETWORKS%

%FILL_VM_TO_NETWORK%

declare -a network_ids
declare -a image_ids

for i in $(seq 0 $((${number_of_networks}-1))); do
  net_list=$(sudo nova-manage network list 2> /dev/null | tail -n+2 | cut --field 2,9)
  if echo "${net_list}" | grep "${networks[${i}]}" &> /dev/null; then
    network_ids[${i}]="$(echo "${net_list}" | grep "${networks[${i}]}" 2> /dev/null | cut --field 2 | sed -e 's/[ \t]*//g')"
    echo "Network ${networks[${i}]} exists with id ${network_ids[${i}]}"
  else
    echo "Creating network ${networks[${i}]}"
    sudo nova-manage network create --label=public --fixed_range_v4="${networks[${i}]}"
    net_list=$(sudo nova-manage network list 2> /dev/null | tail -n+2 | cut --field 2,9)
    network_ids[${i}]="$(echo "${net_list}" | grep "${networks[${i}]}" 2> /dev/null | cut --field 2 | sed -e 's/[ \t][ \t]*//g')"
    echo "Created network ${networks[${i}]} with id ${network_ids[${i}]}"
  fi
done

for i in $(seq 0 $((${number_of_vms}-1))); do
  image_list=$(nova image-list | grep -vE "+--*+--*+--*+--*+" | tail -n+2 | cut --fields=2,3 --delimiter='|' | sed -e 's/\t/ /g' -e 's/|/ /g' | sed -e 's/  */ /g' -e 's/^ *//g')
  if echo "${image_list}" | grep "${images[${i}]}" &> /dev/null; then
    image_ids[${i}]="$(echo "${image_list}" | grep "${images[${i}]}" 2> /dev/null | cut --fields=1 --delimiter=' ' | sed -e 's/[ \t][ \t]*//g')"
    echo "Image ${images[${i}]} exists with id ${image_ids[${i}]}"
    if [ "${demo}" != "true" ]; then
      rm "images/${images[${i}]}"
    fi
  else
    echo "Adding image ${images[${i}]}"
    glance add name="${images[${i}]}" is_public=true container_format=ovf disk_format=qcow2 < "images/${images[${i}]}"
    #rm "images/${images[${i}]}"
    image_list=$(nova image-list | grep -vE "+--*+--*+--*+--*+" | tail -n+2 | cut --fields=2,3 --delimiter='|' | sed -e 's/\t/ /g' -e 's/|/ /g' | sed -e 's/  */ /g' -e 's/^ *//g')
    image_ids[${i}]="$(echo "${image_list}" | grep "${images[${i}]}" 2> /dev/null | cut --fields=1 --delimiter=' ' | sed -e 's/[ \t][ \t]*//g')"
    echo "Created image ${images[${i}]} with id ${image_ids[${i}]}"
  fi
done

for i in $(seq 0 $((${number_of_vms}-1))); do
  vm_name="${vms[${i}]}"
  vm_image="${image_ids[${i}]}"
  vm_networks="${vm_to_network[${i}]}"
  vm_network_line=""
  for n in ${vm_networks}; do
    vm_network_line="${vm_network_line} --nic net-id=${network_ids[${n}]}"
  done
  machine="$(nova list | grep "${vm_name}" | cut --fields=2 --delimiter='|' | sed -e 's/\t/ /g' -e 's/|/ /g' | sed -e 's/  */ /g' -e 's/^ *//g')"
  if [ -n "${machine}" ]; then
    nova delete "${machine}"
  fi
  nova boot --flavor=1 --image="${vm_image}" ${vm_network_line} "${vm_name}"
done

sleep 30
echo "================================================================================"

nova list | grep -vE "+--*+--*+--*+--*+" | tail -n+2 | cut --fields=3,5 --delimiter='|' --output-delimiter=';' | sed -e 's/\t/ /g' -e 's/|/ /g' | sed -e 's/  */ /g' -e 's/^ *//g'
