# encoding: utf-8
# license: gpl3p 

### BEGIN LICENSE NOTICE
# This file is part of the interconnect asset ontology framework (IO)
# Copyright (C) 2010 - 2013
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE NOTICE

### BEGIN AUTHOR LIST
#
### END AUTHOR LIST
require "socket"
require "openssl"
module Iof
  module Control
    class ClientBase < Ghun::Base::Module
      include Ghun::Base::Configuration
      def initialize()
        super(Ghun::Log::Source::CONTROL,Ghun::Log::Source::BASE)
        init_configuration()
        @receiver=nil
        @sender=nil
        @socket=nil
      end

      def connect(socket)
        @socket=socket
        @socket.connect()
      end

      def start_receiver()
        Ghun::Base::Thread.run("Receiver-Thread for #{self.class}##{self.__id__}",0,Ghun::Log::Source::CONTROL) do
          receive()
        end
      end

      def receive()
        begin
          while line_in=@socket.gets
            response=decode(line_in)
            debug Ghun::Log::Type::RECEIVE, "Got message #{response.inspect}"
            puts "Got message #{response.inspect}"
          end
        rescue => e
          error "Failed to receive message from server; aborting receive", e
        end
      end

      def send(request)
        @socket.puts encode(request)
      end

      def decode(line_in)
        request=line_in.chomp
        request=Base64.strict_decode64(request)
        request.force_encoding("UTF-8")
        request=YAML.load(request)
        request
      end

      def encode(response)
        line_out=response.to_yaml()
        line_out=Base64.strict_encode64(line_out)
        line_out
      end
    end

    class Client < ClientBase
      def initialize(config_key_base)
        super()
        config.declare_dynamic_key("#{config_key_base}.host".intern,:string)
        config.declare_dynamic_key("#{config_key_base}.port".intern,:uint,51337)
        config.declare_dynamic_key("#{config_key_base}.ssl".intern,:bool,true)
        config.declare_dynamic_key("#{config_key_base}.certificate".intern,:string,File.join(Ghun::Base::Blackboard.local.data_path,'visa','certs','server.crt'))
        @host=config["#{config_key_base}.host".intern]
        @port=config["#{config_key_base}.port".intern]
        @ssl=config["#{config_key_base}.ssl".intern]
        @ssl_config=Hash.new()
        if @ssl
          @ssl_config[:cert_file]=config["#{config_key_base}.certificate".intern]
          @ssl_config[:cert]=File.read(@ssl_config[:cert_file]) unless @ssl_config[:cert_file].nil?()
          @ssl=!(@ssl_config[:cert].nil?())
        end
      end

      def connect()
        tcp_client=::TCPSocket.new(@host,@port)
        if @ssl
          ssl_server=OpenSSL::SSL::SSLSocket.new(tcp_client)
          ssl_server.sync_close = true
          @server=ssl_server
        else
          @server=tcp_client
        end
        super(@server)
        if @ssl && (@server.peer_cert.to_s != @ssl_config[:cert].to_s) then
          error "Certificate mismatch"
          @server.close()
        end
      end
    end

    class VisaClient < Iof::Control::Client
      def initialize()
        super(:server)
      end

      def decode(line_in)
        Iof::Control::VisaResponse.decode(line_in)
      end

      def encode(request)
        Iof::Control::VisaRequest.encode(request)
      end
    end
  end
end
