# encoding: utf-8
# license: gpl3p 

### BEGIN LICENSE NOTICE
# This file is part of the interconnect asset ontology framework (IO)
# Copyright (C) 2010 - 2013
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE NOTICE

### BEGIN AUTHOR LIST
#
### END AUTHOR LIST
module Iof
  module Ontology
    class Glue < Ghun::Base::Module
      Ghun::Base::Blackboard.config.declare_key(:"ontology.namespace",:array)
      Ghun::Base::Blackboard.config.declare_key(:"ontology.default_namespace",:string)
      def initialize()
        register()
        super(Ghun::Log::Source::ONTOLOGY)
        @sparql_count=0
        @internal_count=0
        @prefixes=Hash.new()
        @default_ns=nil
        @default_ns_url=nil
        reload_configuration()
      end
      attr_reader :default_ns, :default_ns_url, :prefixes

      def reload_configuration()
        @prefixes=Hash.new()
        load_prefixes()
      end

      def set_prefix(ns,url)
        if @prefixes.include?(ns.intern) then
          warn "Namespace '#{ns.to_s}' with url '#{url}' already known"
        else
          @prefixes[ns.intern]=url
        end
      end

      def remove_prefix(ns)
        if @prefixes.include?(ns.intern) then
          debug "Removing namespace '#{ns.to_s}' with url '#{@prefixes[ns.intern]}' already known"
          @prefixes.delete(ns.intern)
        else
          warn "Namespace '#{ns.to_s}' not known"
        end
      end

      def url_to_prefix(url)
        @prefixes.each do |prefix,u|
          return prefix if url==u
        end
        return nil
      end

      def output_counter(message=nil)
        debug Ghun::Log::Type::PROFILING, message unless message.nil?()
        debug Ghun::Log::Type::PROFILING, "Handled #{@sparql_count} SPARQL queries"
        debug Ghun::Log::Type::PROFILING, "Handled #{@internal_count} API calls"
      end

      def reset_counter()
        @sparql_count=0
        @internal_count=0
      end

      def output_and_reset_counter(message=nil)
        output_counter(message)
        reset_counter()
      end

      def syntax_query(query)
        @sparql_count+=1
        return build_query(query)
      end

      def query(query)
        @sparql_count+=1
        return build_query(query)
      end

      def start()
      end

      def finish()
      end

      #FIXME move to other part of IOF, maybe data handler or data mapper
      def search_by_identifier(identifer,klass)
        @internal_count+=1
      end

      def create_individual(klass,individual,klass_ns,individual_ns)
        @internal_count+=1
      end

      def exists_individual?(individual,individual_ns)
        @internal_count+=1
      end

      def drop_individual(individual,individual_ns)
        @internal_count+=1
      end

      def has_datatype_property?(individual,property,individual_ns,property_ns)
        @internal_count+=1
      end

      def has_object_property?(individual,property,individual_ns,property_ns)
        @internal_count+=1
      end

      def get_all_datatype_properties(individual,blacklist,individual_ns,list_namespaces,show_unknown_nodes)
        @internal_count+=1
      end

      def get_all_object_properties(individual,blacklist,individual_ns,list_namespaces,show_unknown_nodes)
        @internal_count+=1
      end

      def drop_all_properties(individual,individual_ns)
        @internal_count+=1
      end

      def get_object_property(individual,property,individual_ns,property_ns,list_namespaces,show_unknown_nodes)
        @internal_count+=1
      end

      def get_datatype_property(individual,property,individual_ns,property_ns)
        @internal_count+=1
      end

      def drop_property(individual,property,individual_ns,property_ns)
        @internal_count+=1
      end

      def drop_object_property(individual,property,individual_ns,property_ns)
        @internal_count+=1
      end

      def drop_datatype_property(individual,property,individual_ns,property_ns)
        @internal_count+=1
      end

      def create_object_property(individual,property,target,individual_ns,property_ns,target_ns)
        @internal_count+=1
      end

      def create_datatype_property(individual,property,value,individual_ns,property_ns)
        @internal_count+=1
      end

    private

      def load_prefixes()
        namespaces=config[:"ontology.namespace"]
        default=config[:"ontology.default_namespace"]
        namespaces.each do |ns|
          config.declare_dynamic_key("ontology.namespace.#{ns.to_s}".intern,:string)
          set_prefix(ns, config["ontology.namespace.#{ns.to_s}".intern])
        end
        @default_ns=default.intern
        @default_ns_url=@prefixes[default.intern]
      end

      def build_query(query)
        q=""
        @prefixes.each do |ns,url|
          q+="PREFIX #{ns.to_s}:\t<#{url}>\r\n"
        end unless @prefixes.nil?()
        q+="\r\n"
        q+=query
        return q
      end

      def format_result(result_raw,list_namespaces,show_unknown_nodes)
        list_namespaces=@prefixes.keys if list_namespaces.nil?() || list_namespaces.empty?()
        return nil unless result_raw
        debug "Formatting result"
        result=Array.new()
        result_raw.each do |r|
          abort=false
          result_line=Hash.new
          r.each do |field,value|
            next if value.nil?()
            found=false
            @prefixes.each do |prefix,url|
              if value.match(/^#{url}/) then
                found=true
                if list_namespaces.include?(prefix) then
                  value.gsub!(/^#{url}/,'')
                  result_line[field]=value
                  result_line["__ns_#{field.to_s}".intern]=prefix
                  result_line["__unknown_#{field.to_s}".intern]=false
                else
                  debug "Value '#{prefix}:#{value}' matched by namespace filter"
                  abort=true
                  break
                end
              end
            end
            break if abort
            if show_unknown_nodes then
              debug "Adding result '#{value}' with unknown namespace to result"
              result_line[field]=value
              result_line["__unknown_#{field.to_s}".intern]=true
            else
              debug "Result '#{value}' contains unknown namespace"
              abort=true
              break
            end unless found
          end
          next if abort
          result << result_line unless result_line.nil?() || result_line.empty?()
        end unless result_raw.nil?()
        return result
      end
    end
  end
end
