# encoding: utf-8
# license: gpl3p

### BEGIN LICENSE NOTICE
# This file is part of the interconnect asset ontology framework (IO)
# Copyright (C) 2010 - 2013
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE NOTICE

### BEGIN AUTHOR LIST
#
### END AUTHOR LIST
Ghun::Base.java_require('pellet')

module Iof
  module Ontology
    class JenaGlue < Iof::Ontology::Glue
      #TODO interface is not thread safe
      def initialize(use_tdb=false,safe_mode=true,load_new=false,ontology_file=nil)
        super()
        @safe_mode=safe_mode
        @load_new=load_new
        @safe_mode=false if @load_new
        @use_tdb=use_tdb
        options=Java::Util::Properties.new()
        options.set_property("USE_UNIQUE_NAME_ASSUMPTION","true")
        options.set_property("REALIZE_INDIVIDUAL_AT_A_TIME","true")
        options.set_property("USE_ANNOTATION_SUPPORT","true")
        options.set_property("HIDE_TOP_PROPERTY_VALUES","false")
        Pellet::PelletOptions.set_options(options)
        #TODO migrate to config
        @ontology_file_structure=File.join(Ghun::Base::Blackboard.application.data_path,'ontologies','network.owl')
        if ontology_file.nil?() then
          @ontology_file_init=File.join(Ghun::Base::Blackboard.application.data_path,'ontologies','network.owl')
          @ontology_file_store=File.join(Ghun::Base::Blackboard.local.data_path,'ontologies','network.owl')
          @ontology_file_read=File.join(Ghun::Base::Blackboard.local.data_path,'ontologies','network.owl')
        else
          @ontology_file_init=ontology_file
          @ontology_file_store=ontology_file
          @ontology_file_read=ontology_file
        end
        @base=nil
        @model=nil
        @structure_model=nil
        @working_without_reasoner=false
        #XXX migrate tdb usage
        @use_tdb=false
        if @use_tdb then
          @tdb_ontology_dir="#{$APP_PATH}/../data/ontologies/tdb"
          unless File.exists?("#{@tdb_ontology_dir}") then
            info "Creating TDB file structure from owl file"
            Dir.mkdir(@tdb_ontology_dir, 0755)
            Jena::Tdb::TdbLoader.main("--loc","#{@tdb_ontology_dir}","#{@ontology_file_init}")
          end
        else
          if load_new then
            @ontology_file=@ontology_file_store
          else
            @ontology_file=@ontology_file_init
          end
          FileUtils.cp(@ontology_file_structure,@ontology_file) unless File.exists?(@ontology_file)
        end
        @datatype_properties=nil
        @object_properties=nil
        @classes=nil
      end
      attr_reader :datatype_properties, :object_properties, :classes, :model

      def syntax_query(query,show_unknown_nodes=false,list_namespaces=[@default_ns])
        list_namespaces=@prefixes.keys if list_namespaces.nil?() || list_namespaces.empty?()
        prefixed_query=super(query)
        debug "Executing query '#{prefixed_query}', with namespace filter '#{list_namespaces.inspect}', without pellet"
        result=jena_query(@structure_model,prefixed_query,show_unknown_nodes,list_namespaces)
        debug "Query finished"
        return result
      rescue => e
        error "Failed to execute query: '#{prefixed_query}'", e
        return nil
      end

      def query(query,show_unknown_nodes=false,list_namespaces=[@default_ns])
        list_namespaces=@prefixes.keys if list_namespaces.nil?() || list_namespaces.empty?()
        prefixed_query=super(query)
        debug "Executing query '#{prefixed_query}', with namespace filter '#{list_namespaces.inspect}'"
        result=jena_query(@model,prefixed_query,show_unknown_nodes,list_namespaces)
        debug "Query finished"
        return result
      rescue => e
        error "Failed to execute query: '#{prefixed_query}'", e
        return nil
      end

      def start_for_mapping
        read_ontology_for_mapping
        return nil
      end

      def start_for_writing
        @working_without_reasoner=true
        read_ontology_for_writing
        return nil
      end

      def start
        @working_without_reasoner=false
        read_ontology
        debug "Preparing ontology"
        @model.prepare
        @graph=@model.get_graph
        debug "Classifying ontology"
        @graph.classify
        debug "Realizing ontology"
        @graph.realize
        return nil
      end

      def finish(read_only=false)
        write_ontology unless read_only
        @model.close()
        @model=nil
        return nil
      end

      def finish_mapping()
        @structure_model.close()
        @structure_model=nil
        return nil
      end


      def create_individual(klass,individual,klass_ns=@default_ns,individual_ns=@default_ns)
        super(klass,individual,klass_ns.intern,individual_ns.intern)
        c=get_jena_class(klass,klass_ns.intern)
        @model.create_individual("#{@prefixes[individual_ns.intern]}#{individual}",c)
        return { :name => individual, :ns => individual_ns.intern, :unknown => false }
      rescue => e
        error "Failed to create individual '#{individual_ns.to_s}:#{individual}' with class '#{klass_ns.to_s}:#{klass}'", e
        return nil
      end

      def exists_individual?(individual,individual_ns=@default_ns)
        super(individual,individual_ns.intern)
        !get_jena_individual(individual,individual_ns.intern).nil?()
      end

      def drop_individual(individual,individual_ns=@default_ns)
        super(individual,individual_ns.intern)
        i=get_jena_individual(individual,individual_ns.intern)
        if i.nil?() then
          error "Individual '#{individual_ns.to_s}:#{individual}' does not exist"
          return nil
        end
        i.remove()
        return true
      rescue => e
        error "Could not remove individual '#{individual_ns.to_s}:#{individual}'", e
        return false
      end

      def has_datatype_property?(individual,property,individual_ns=@default_ns,property_ns=@default_ns)
        super(individual,property,individual_ns.intern,property_ns.intern)
        p=get_jena_datatype_property(property,property_ns.intern)
        if p.nil?() then
          error "Datatype property '#{property_ns.to_s}:#{property}' is unknown"
          return nil
        end
        i=get_jena_individual(individual,individual_ns.intern)
        if i.nil?() then
          error "Individual '#{individual_ns.to_s}:#{individual}' does not exist"
          return nil
        end
        return i.has_property(p)
      rescue => e
        error "Could not determine existence of datatype property '#{property_ns.to_s}:#{property}' of individual '#{individual_ns.to_s}:#{individual}'", e
        return nil
      end

      def has_object_property?(individual,property,individual_ns=@default_ns,property_ns=@default_ns)
        super(individual,property,individual_ns.intern,property_ns.intern)
        p=get_jena_object_property(property,property_ns.intern)
        if p.nil?() then
          error "Object property '#{property_ns.to_s}:#{property}' is unknown"
          return nil
        end
        i=get_jena_individual(individual,individual_ns.intern)
        if i.nil?() then
          error "Individual '#{individual_ns.to_s}:#{individual}' does not exist"
          return nil
        end
        return i.has_property?(p)
      rescue => e
        error "Could not determine existence of object property '#{property_ns.to_s}:#{property}' of individual '#{individual_ns.to_s}:#{individual}'", e
        return nil
      end

      def get_all_datatype_properties(individual,blacklist=[],individual_ns=@default_ns,list_namespaces=[@default_ns],show_unknown_nodes=false)
        list_namespaces=@prefixes.keys if list_namespaces.nil?() || list_namespaces.empty?()
        super(individual,blacklist,individual_n.interns,list_namespaces,show_unknown_nodes)
        i=get_jena_individual(individual,individual_ns.intern)
        if i.nil?() then
          error "Individual '#{individual_ns.to_s}:#{individual}' does not exist"
          return nil
        end
        properties=Hash.new()
        statements=i.list_properties.to_a
        statements.each do |s|
          p=s.get_predicate.to_s
          next if p.nil?()
          found=false
          type_match=false
          property_prefix=nil
          @prefixes.each do |prefix,url|
            if p.match(/^#{url}/) then
              found=true
              property_prefix=prefix
              if list_namespaces.include?(prefix) && @data_properties[prefix].include?(p.intern) && !(blacklist.include?(prefix) && blacklist[prefix].include?(p.intern)) then
                type_match=true
                p.gsub!(/^#{url}/,'')
                unless properties.include?("#{prefix}:#{p}".intern) then
                  properties["#{prefix}:#{p}".intern]=Hash.new
                  properties["#{prefix}:#{p}".intern][:name]=p
                  properties["#{prefix}:#{p}".intern][:ns]=prefix
                  properties["#{prefix}:#{p}".intern][:unknown]=false
                  properties["#{prefix}:#{p}".intern][:values]=Array.new()
                end
              else
                debug "Property '#{prefix}:#{p}' matched by namespace filter" unless list_namespaces.include?(prefix)
                debug "Property '#{prefix}:#{p}' is not an data property" unless @data_properties[prefix].include?(p.intern)
                debug "Property '#{p}' for individual '#{individual_ns.to_s}:#{individual}' is blacklisted by caller" if blacklist.include?(prefix) && blacklist[prefix].include?(p.intern)
              end
            end
          end
          if show_unknown_nodes then
            debug "Adding property '#{p}' with unknown namespace to result"
            properties["#{p}".intern]=Hash.new
            properties["#{p}".intern][:property]=p
            properties["#{p}".intern][:unknown]=true
            properties["#{p}".intern][:values]=Array.new()
          else
            debug "Property '#{p}' contains unknown namespace"
          end unless found
          if type_match then
            value=s.get_literal.to_string
            properties["#{property_prefix}:#{p}".intern][:values] << { :value => value }
          end
        end unless statements.nil?()
        return properties
      rescue => e
        error "Could not retrieve list of all object properties for individual '#{individual_ns.to_s}:#{individual}'", e
        return nil
      end

      def get_all_object_properties(individual,blacklist=[],individual_ns=@default_ns,list_namespaces=[@default_ns],show_unknown_nodes=false)
        list_namespaces=@prefixes.keys if list_namespaces.nil?() || list_namespaces.empty?()
        super(individual,blacklist,individual_ns.intern,list_namespaces,show_unknown_nodes)
        i=get_jena_individual(individual,individual_ns.intern)
        if i.nil?() then
          error "Individual '#{individual_ns.to_s}:#{individual}' does not exist"
          return nil
        end
        properties=Hash.new()
        statements=i.list_properties.to_a
        statements.each do |s|
          p=s.get_predicate.to_s
          next if p.nil?()
          found=false
          type_match=false
          property_prefix=nil
          @prefixes.each do |prefix,url|
            if p.match(/^#{url}/) then
              found=true
              property_prefix=prefix
              if list_namespaces.include?(prefix) && @object_properties[prefix].include?(p.intern) && !(blacklist.include?(prefix) && blacklist[prefix].include?(p.intern)) then
                type_match=true
                p.gsub!(/^#{url}/,'')
                unless properties.include?("#{prefix}:#{p}".intern) then
                  properties["#{prefix}:#{p}".intern]=Hash.new
                  properties["#{prefix}:#{p}".intern][:property]=p
                  properties["#{prefix}:#{p}".intern][:ns]=prefix
                  properties["#{prefix}:#{p}".intern][:unknown]=false
                  properties["#{prefix}:#{p}".intern][:values]=Array.new()
                end
              else
                debug "Property '#{prefix}:#{p}' matched by namespace filter" unless list_namespaces.include?(prefix)
                debug "Property '#{prefix}:#{p}' is not an object property" unless @object_properties[prefix].include?(p.intern)
                debug "Property '#{p}' for individual '#{individual_ns.to_s}:#{individual}' is blacklisted by caller" unless blacklist.include?(prefix) && blacklist[prefix].include?(p.intern)
              end
            end
          end
          if show_unknown_nodes then
            debug "Adding property '#{p}' with unknown namespace to result"
            properties["#{p}".intern]=Hash.new
            properties["#{p}".intern][:property]=p
            properties["#{p}".intern][:unknown]=true
            properties["#{p}".intern][:values]=Array.new()
          else
            debug "Property '#{p}' contains unknown namespace"
          end unless found
          if type_match then
            found=false
            value=s.get_resource.to_string
            @prefixes.each do |prefix,url|
              if value.match(/^#{url}/) then
                found=true
                if list_namespaces.include?(prefix) then
                  value.gsub!(/^#{url}/,'')
                  properties["#{property_prefix}:#{p}".intern][:values] << { :name => value, :ns => prefix, :unknown => false }
                else
                  debug "Value '#{prefix}:#{value}' matched by namespace filter"
                end
              end
            end
            if show_unknown_nodes then
              debug "Adding result '#{value}' with unknown namespace to result"
              if property_prefix.nil?() then
                properties[p.intern] << value
              else
                properties["#{property_prefix}:#{p}".intern][:values] << { :name => value, :unknown => true }
              end
            else
              debug "Result '#{value}' contains unknown namespace"
            end unless found
          end
        end unless statements.nil?()
        return properties
      rescue => e
        error "Could not retrieve list of all object properties for individual '#{individual_ns.to_s}:#{individual}'", e
        return nil
      end

      def drop_all_properties(individual,individual_ns=@default_ns)
        super(individual,individual_ns.intern)
        i=get_jena_individual(individual,individual_ns.intern)
        if i.nil?() then
          error "Individual '#{individual_ns.to_s}:#{individual}' does not exist"
          return nil
        end
        i.clear_properties
        return true
      rescue => e
        error "Could not remove all properties for individual '#{individual_ns.to_s}:#{individual}'",e
        return false
      end

      def get_object_property(individual,property,individual_ns=@default_ns,property_ns=@default_ns,list_namespaces=[@default_ns],show_unknown_nodes=false)
        list_namespaces=@prefixes.keys if list_namespaces.nil?() || list_namespaces.empty?()
        super(individual,property,individual_ns.intern,property_ns.intern,list_namespaces,show_unknown_nodes)
        result=Array.new()
        p=get_jena_object_property(property,property_ns.intern)
        if p.nil?() then
          error "Object property '#{property_ns.to_s}:#{property}' is unknown"
          return nil
        end
        i=get_jena_individual(individual,individual_ns.intern)
        if i.nil?() then
          error "Individual '#{individual_ns.to_s}:#{individual}' does not exist"
          return nil
        end
        values=i.list_property_values(p)
        values.each do |value|
          found=false
          @prefixes.each do |prefix,url|
            if value.to_string.match(/^#{url}/) then
              v=value.to_string
              found=true
              if list_namespaces.include?(prefix) then
                v.gsub!(/^#{url}/,'')
                r=Hash.new()
                r[:name]=v
                r[:ns]=prefix
                r[:unknown]=false
                result << r
              else
                debug "Value #{value.to_string} matched by namespace filter"
              end
            end
          end
          if show_unknown_nodes then
            debug "Adding result '#{value.to_string}' with unknown namespace to result"
            r=Hash.new()
            r[:name]=value.to_string
            r[:unknown]=true
            result << r
          else
            debug "Result '#{value.to_string}' contains unknown namespace"
          end unless found
        end unless values.nil?
        return result
      rescue => e
        error "Could not determine value(s) of property '#{property_ns.to_s}:#{property}' of individual '#{individual_ns.to_s}:#{individual}'", e
        return nil
      end

      def get_datatype_property(individual,property,individual_ns=@default_ns,property_ns=@default_ns)
        super(individual,property,individual_ns.intern,property_ns.intern)
        result=Array.new()
        p=get_jena_datatype_property(property.intern,property_ns)
        if p.nil?() then
          error "Datatype property '#{property_ns.to_s}#{property}' is unknown"
          return nil
        end
        i=get_jena_individual(individual,individual_ns.intern)
        if i.nil?() then
          error "Individual '#{individual_ns.to_s}:#{individual}' does not exist"
          return nil
        end
        values=i.list_property_values(p)
        values.each do |v|
          result << v.get_string
        end unless values.nil?
        return result
      rescue => e
        error "Could not determine value(s) of property '#{property_ns.to_s}:#{property}' of individual '#{individual_ns.to_s}:#{individual}'", e
        return nil
      end

      def drop_property(individual,property,individual_ns=@default_ns,property_ns=@default_ns)
        super(individual,property,individual_ns.intern,property_ns.intern)
        i=get_jena_individual(individual,individual_ns.intern)
        if i.nil?() then
          error "Individual '#{individual_ns.to_s}:#{individual}' does not exist"
          return nil
        end
        prefix=nil
        if property_ns.is_a?(Symbol) then
          prefix=property_ns.intern
        else
          prefix=url_to_prefix(property_ns)
        end
        p=nil
        p=get_jena_datatype_property(property,property_ns.intern) if @datatype_properties.include?(prefix) && @datatype_properties[prefix].include?(property)
        p=get_jena_object_property(property,property_ns.intern) if @object_properties.include?(prefix) && @object_properties[prefix].include?(property)
        if p.nil?() then
          error "Property '#{property_ns.to_s}:#{property}' is unknown"
          return nil
        end
        i.remove_all(p)
        return true
      rescue => e
        error "Could not remove property '#{property_ns.to_s}:#{property}' for individual '#{individual_ns.to_s}:#{individual}'", e
        return false
      end

      def drop_object_property(individual,property,individual_ns=@default_ns,property_ns=@default_ns)
        super(individual,property,individual_ns.intern,property_ns.intern)
        i=get_jena_individual(individual,individual_ns.intern)
        if i.nil?() then
          error "Individual '#{individual_ns.to_s}:#{individual}' does not exist"
          return nil
        end
        p=get_jena_object_property(property,property_ns.intern)
        if p.nil?() then
          error "Object property '#{property_ns.to_s}:#{property}' is unknown"
          return nil
        end
        i.remove_all(p)
        return true
      rescue => e
        error "Could not remove property '#{property_ns.to_s}:#{property}' for individual '#{individual_ns.to_s}:#{individual}'", e
        return false
      end

      def drop_datatype_property(individual,property,individual_ns=@default_ns,property_ns=@default_ns)
        super(individual,property,individual_ns.intern,property_ns.intern)
        i=get_jena_individual(individual,individual_ns.intern)
        if i.nil?() then
          error "Individual '#{individual_ns.to_s}:#{individual}' does not exist"
          return nil
        end
        p=get_jena_datatype_property(property,property_ns.intern)
        if p.nil?() then
          error "Datatype property '#{property_ns.to_s}:#{property}' is unknown"
          return nil
        end
        i.remove_all(p)
        return true
      rescue => e
        error "Could not remove property '#{property_ns.to_s}:#{property}' for individual '#{individual_ns.to_s}:#{individual}'", e
        return false
      end

      def create_object_property(individual,property,target,individual_ns=@default_ns,property_ns=@default_ns,target_ns=@default_ns)
        super(individual,property,target,individual_ns.intern,property_ns.intern,target_ns.intern)
        i=get_jena_individual(individual,individual_ns.intern)
        p=get_jena_object_property(property,property_ns.intern)
        t=get_jena_individual(target,target_ns.intern)
        error "Failed to retrieve jena object for individual '#{individual_ns.to_s}:#{individual}'" if i.nil?()
        error "Failed to retrieve jena object for property '#{property_ns.to_s}:#{property}'" if p.nil?()
        error "Failed to retrieve jena object for individual '#{target_ns.to_s}:#{target}'" if t.nil?()
        s=@model.createStatement(i,p,t)
        @model.add(s)
        return true
      rescue => e
        error "Failed to create object property '#{property_ns.to_s}:#{property}' between '#{individual_ns.to_s}:#{individual}' and '#{target_ns.to_s}:#{target}'", e
        return false
      end

      def create_datatype_property(individual,property,value,individual_ns=@default_ns,property_ns=@default_ns)
        super(individual,property,value,individual_ns.intern,property_ns.intern)
        i=get_jena_individual(individual,individual_ns.intern)
        p=get_jena_datatype_property(property,property_ns.intern)
        error "Failed to retrieve jena object for individual '#{individual_ns.to_s}:#{individual}'" if i.nil?()
        error "Failed to retrieve jena object for property '#{property_ns.to_s}:#{property}'" if p.nil?()
        s=@model.createStatement(i,p,value.to_s)
        @model.add(s)
        return true
      rescue => e
        error "Failed to create datatype property '#{property_ns.to_s}:#{property}' for '#{individual_ns.to_s}:#{individual}' with value '#{value}'", e
        return false
      end

    private

      def jena_query(model,query,show_unknown_nodes,list_namespaces)
        list_namespaces=@prefixes.keys if list_namespaces.nil?() || list_namespaces.empty?()
        results=Array.new()
        exec = Jena::Query::QueryExecutionFactory.create(Jena::Query::QueryFactory.create(query), model)
        begin
          exec.execSelect.each do |raw|
            result=Hash.new()
            vars=raw.varNames.to_a
            vars.each do |v|
              r=raw.get(v)
              if r.nil?() then
                result[v.intern] = r
              else
                result[v.intern] = r.to_string
              end
            end
            results << result
          end
        rescue => e
          error "Failed to execute query: '#{query}'", e
          results=nil
        ensure
          exec.close
        end
        return format_result(results,list_namespaces,show_unknown_nodes)
      end


      def get_jena_individual(name,ns)
        prefix=nil
        if ns.is_a?(Symbol) then
          prefix=ns
        else
          prefix=url_to_prefix(ns)
        end
        @model.get_individual("#{@prefixes[prefix]}#{name}")
      end

      def get_jena_datatype_property(name,ns)
        prefix=nil
        if ns.is_a?(Symbol) then
          prefix=ns
        else
          prefix=url_to_prefix(ns)
        end
        if @datatype_properties.include?(prefix) && @datatype_properties[prefix].include?(name.intern) then
          return @datatype_properties[prefix][name.intern]
        else
          error "Datatype property '#{name}' with namespace #{ns} unknown in ontology"
          raise PropertyUnknownError
        end
      end

      def get_jena_object_property(name,ns)
        prefix=nil
        if ns.is_a?(Symbol) then
          prefix=ns
        else
          prefix=url_to_prefix(ns)
        end
        if @object_properties.include?(prefix) && @object_properties[prefix].include?(name.intern) then
          return @object_properties[prefix][name.intern]
        else
          error "Object property '#{name}' with namespace #{ns} unknown in ontology"
          raise PropertyUnknownError
        end
      end

      def get_jena_class(name,ns)
        prefix=nil
        if ns.is_a?(Symbol) then
          prefix=ns
        else
          prefix=url_to_prefix(ns)
        end
        if @classes.include?(prefix) && @classes[prefix].include?(name.intern) then
          return @classes[prefix][name.intern]
        else
          error "Class '#{name}' with namespace #{ns} unknown in ontology"
          raise ClassUnknownError, "Class '#{name}' with namespace #{ns} unknown in ontology"
        end
      end

      def read_ontology_for_writing()
        if @use_tdb then
          raw_model=Jena::Tdb::TDBFactory.createModel(@tdb_ontology_dir)
          @model = Jena::Rdf::Model::ModelFactory.createOntologyModel(Jena::Ontology::OntModelSpec::OWL_MEM, raw_model)
        else
          raw_model=Jena::Rdf::Model::ModelFactory.createDefaultModel()
          raw_model.read("file://#{@ontology_file}")
          @model = Jena::Rdf::Model::ModelFactory.createOntologyModel(Jena::Ontology::OntModelSpec::OWL_MEM, raw_model)
        end
        init_classes(@model)
        init_properties(@model)
      end

      def read_ontology_for_mapping()
        error "TDB is not supported for syntax queries"  if @use_tdb
        model=Jena::Rdf::Model::ModelFactory.createOntologyModel( Pellet::Jena::PelletReasonerFactory.THE_SPEC )
        model.read("file://#{@ontology_file_structure}")
        debug "Preparing ontology"
        model.prepare
        graph=model.get_graph
        debug "Classifying ontology"
        graph.classify
        debug "Realizing ontology"
        graph.realize
        @structure_model=Jena::Rdf::Model::ModelFactory.createOntologyModel(Jena::Ontology::OntModelSpec::OWL_MEM)
        @structure_model.add(model)
        init_classes(@structure_model)
        init_properties(@structure_model)
      end

      def read_ontology()
        if @use_tdb then
          raw_model=Jena::Tdb::TDBFactory.createModel(@tdb_ontology_dir)
          @model = Jena::Rdf::Model::ModelFactory.createOntologyModel(Pellet::Jena::PelletReasonerFactory.THE_SPEC, raw_model);
        else
          @model=Jena::Rdf::Model::ModelFactory.createOntologyModel( Pellet::Jena::PelletReasonerFactory.THE_SPEC )
          @model.read("file://#{@ontology_file_read}")
        end
        init_classes(@model)
        init_properties(@model)
      end

      def write_ontology()
        @model.commit() if @use_tdb
        io=Java::IO::FileOutputStream.new(@ontology_file_store)
        @model.write(io)
        io.close()
      end

      def close_ontology()
        @model.close
        @model=nil
      end


      def init_classes(model)
        @classes=Hash.new()
        @prefixes.keys.each do |prefix|
          @classes[prefix]=Hash.new()
        end
        model.list_classes.each do |c|
          @prefixes.each do |prefix,url|
            if c.to_string.match(/^#{url}/) then
              debug "Found class '#{c.to_string}' with namespace '#{prefix.to_s}' in ontology"
              @classes[prefix][c.to_string.sub(/^#{url}/,"").intern]=c
              break
            end
          end
        end unless model.nil?
      end

      def init_properties(model)
        @datatype_properties=Hash.new()
        @object_properties=Hash.new
        @prefixes.keys.each do |prefix|
          @datatype_properties[prefix]=Hash.new()
          @object_properties[prefix]=Hash.new
        end
        model.list_object_properties.each do |op|
          @prefixes.each do |prefix,url|
            if op.to_string.match(/^#{url}/) then
              debug "Found object property '#{op.to_string}' with namespace '#{prefix.to_s}' in ontology"
              @object_properties[prefix][op.to_string.sub(/^#{url}/,"").intern]=op
              break
            end
          end
        end unless model.nil?
        model.list_datatype_properties.each do |dp|
          @prefixes.each do |prefix,url|
            if dp.to_string.match(/^#{url}/) then
              debug "Found datatype property '#{dp.to_string}' with namespace '#{prefix.to_s}' in ontology"
              @datatype_properties[prefix][dp.to_string.sub(/^#{url}/,"").intern]=dp
              break
            end
          end
        end unless model.nil?
      end
    end
  end
end
