# encoding: utf-8
# license: gpl3p 

### BEGIN LICENSE NOTICE
# This file is part of the interconnect asset ontology framework (IO)
# Copyright (C) 2010 - 2013
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE NOTICE

### BEGIN AUTHOR LIST
#
### END AUTHOR LIST
module Iof
  module Parser
    class Openstack < Iof::Parser::Base
      Iof::Parser::SELECTOR.register_member(self,[:visa,:ovs,:ssh,2])
      def initialize(target,data_handler,target_handler)
        super(target,data_handler,target_handler)
        @vms=Hash.new()
        @images=Hash.new()
        @interfaces=Hash.new()
        @host=Hash.new()
        @config=Hash.new()
        @results=Hash.new()

        @bridges=Hash.new()
        @ports=Hash.new()
        @ovs_interfaces=Hash.new()
      end

      def parse
        unless super()
          @target.status_parse=false
          return false
        end
        [:ifconfig,:ovs_vs_list_bridge,:ovs_vs_list_port,:ovs_vs_list_interface,:ovs_vs_show,:nova_list,:nova_details,:nova_create_images,:nova_config].each do |label|
#          next unless @target.commands.keys.include?(label)
          parse_command(label,:parse)
        end
        parse_target
      rescue => e
        error "Failed to parse target #{@target.description()}", e
        @target.status_parse=false
        return false
      end

      def postparse()
        unless super()
          @target.status_postparse=false
          return false
        end
        if @target.device_type==:plain then
          postprocess_parsed_data_plain
        else
          postprocess_parsed_data_openvswitch
        end
        send_results
      rescue => e
        error "Failed to postparse target #{@target.description()}", e
        @target.status_postparse=false
        return false
      end

    private

      def parse_target()
        @host[:name]=@target.names[0]
        @host[:management_address]=@target.ipv4_addresses[0]
        @host[:vsa]=@target.special[:vsa]
        @host[:is_virtualisation_host]=true
        @host[:is_endpoint]=true
        @host[:is_virtual_machine]=false
        if @target.device_type==:plain then
          @host[:network_structure]=:plain
        elsif @target.device_type==:ovs then
          @host[:network_structure]=:openvswitch
        else
          @host[:network_structure]=:none
        end
      end

      def parse_ifconfig(label,command,result,step)
        result[:stdout].split(/\n\n/).each do |block|
          interface=Hash.new()
          first_line=0
          block.split(/\n/).each do |line|
            if first_line==0 then
              sp=line.split(' ')
              interface[:name]=sp[0]
              (1..(sp.size()-1)).each do |i|
                if sp[i].match(/encap/) then
                  interface[:type]=sp[i].sub(/encap:/,'').downcase()
                elsif sp[i].match(/HWaddr/) then
                  interface[:l2_address]=sp[i+1]
                end
              end
              first_line+=1
            else
              if line.match(/inet addr/) then
                addr=line.sub(/ Mask.*/,'').sub(/ Bcast.*/,'').sub(/inet addr:/,'').strip
                mask=line.sub(/[^M]*Mask:/,'').strip
                length=Iof::Data.ipv4_netmask_to_length(mask)
                interface[:ipv4]=Hash.new unless interface.include?(:ipv4)
                interface[:ipv4][:addr]=Array.new() unless interface[:ipv4].include?(:addr)
                interface[:ipv4][:net]=Array.new() unless interface[:ipv4].include?(:net)
                interface[:ipv4][:net_raw]=Array.new() unless interface[:ipv4].include?(:net_raw)
                interface[:ipv4][:addr] << @data_handler.get_ipv4_address_name(addr, length)
                net=Iof::Data.ipv4_network_part(addr,length)
                interface[:ipv4][:net] << @data_handler.get_ipv4_network_name(net, length)
                interface[:ipv4][:net_raw] << IPAddress::IPv4.new("#{net}/#{length}")
              elsif line.match(/inet6 addr/) then
                addr=line.sub(/[ \t]*inet6 addr:/,'').sub(/\/.*/,'').strip
                length=line.sub(/.*\//,'').sub(/Scope:.*/,'').strip
                interface[:ipv6]=Hash.new unless interface.include?(:ipv6)
                interface[:ipv6][:addr]=Array.new() unless interface[:ipv6].include?(:addr)
                interface[:ipv6][:net]=Array.new() unless interface[:ipv6].include?(:net)
                interface[:ipv6][:net_raw]=Array.new() unless interface[:ipv6].include?(:net_raw)
                interface[:ipv6][:addr] << @data_handler.get_ipv6_address_name(addr, length)
                net=Iof::Data.ipv6_network_part_uncompressed(addr,length)
                interface[:ipv6][:net] << @data_handler.get_ipv6_network_name(net, length)
                interface[:ipv6][:net_raw] << IPAddress::IPv6.new("#{net}/#{length}")
              else
                debug "ignoring line '#{line}' for interface '#{interface[:name]}'"
              end
            end
          end
          if interface.include?(:name) then
            debug "Finished parsing for interface '#{interface[:name]}'"
            @interfaces[interface[:name].intern]=interface
          else
            error "Finished parsing for interface without name '#{interface.inspect}'; skipping"
          end
        end
      end

      def parse_route_ipv4(label,command,result,step)
        skip=0
        result[:stdout].split("\n").each do |line|
          skip+=1
          next if skip<=2
          sp=line.split(' ')
          route=Hash.new()
          route[:destination]=@data_handler.get_ipv4_network_name(sp[0], sp[2])
          route[:gateway]=@data_handler.get_ipv4_address_name(sp[1])
          route[:interface]=sp[7]
          @routes_ipv4 << route
        end
      end

      def parse_route_ipv6(label,command,result,step)
        unless result[:stderr].nil?() || result [:stderr]==""
          error "STDERR is not empty in result for command with label #{label} ('#{command}')", "Got #{result[:stderr]}"
          raise UnexpectedResultError
        end
        if result[:stdout].nil?
          error "STDOUT is empty in result for command with label #{label} ('#{command}')"
          raise UnexpectedResultError
        end

        skip=0
        result[:stdout].split("\n").each do |line|
          skip+=1
          next if skip<=2
          sp=line.split(' ')
          route=Hash.new()
          route[:destination]=@data_handler.get_ipv6_network_name(sp[0].sub(/\/.*/,''),sp[0].sub(/.*\//,''))
          route[:next_hop]=@data_handler.get_ipv6_address_name(sp[1])
          route[:interface]=sp[6]
          @routes_ipv6 << route
        end
      end

      def is_garbage?(line)
        return true if line.nil?()
        return true if line==""
        return true if line.match(/\+-+\+-+\+-+\+-+\+/)
        return true if line.match(/\+-+\+-+\+/)
      end

      def parse_nova_list(label,command,result,step)
        skip=0
        result[:stdout].split("\n").each do |line|
          next if is_garbage?(line)
          skip+=1
          next if skip==1
          line.strip!
          line_split=line.split('|')[1..5]
          line_split.each do |field|
            field.strip!
          end
          if @vms.include?(line_split[0].intern) then
            error "Got duplicate VMs id '#{line_split[0]}' for VM with name '#{line_split[1]}'; skipping line"
            next
          end
          @vms[line_split[0].intern]=Hash.new
          @vms[line_split[0].intern][:id]=line_split[0]
          @vms[line_split[0].intern][:name]=line_split[1]
          @vms[line_split[0].intern][:status]=line_split[2].downcase
          @vms[line_split[0].intern][:vsa]=@target.special[:vsa]
          @vms[line_split[0].intern][:is_virtualisation_host]=false
          @vms[line_split[0].intern][:is_virtual_machine]=true
          @vms[line_split[0].intern][:is_endpoint]=true
          #Networks are read from nova show <id> output
        end
      end

      def parse_nova_details(label,command,result,step)
        skip=0
        entry=Hash.new()
        result[:stdout].split("\n").each do |line|
          next if is_garbage?(line)
          skip+=1
          if line.match(/^=+$/) then
            if entry.include?(:id) then
              debug "Finished detail entry for VM with id '#{entry[:id]}'"
              if @vms.include?(entry[:id].intern) then
                @vms[entry[:id].intern].merge!(entry)
              else
                @vms[entry[:id].intern]=entry
              end
            else
              error "Read vm detail entry without id; discarding data", entry.inspect
            end
            debug "Starting new detail entry"
            entry=Hash.new()
            skip=0
            next
          end
          next if skip==1
          line.strip!
          line_split=line.split('|')[1..2]
          line_split.each do |field|
            field.strip!
          end
          case line_split[0]
          when "user_id"
            debug "Ignoring field '#{line_split[0]}' with value '#{line_split[1]}'"
          when "tenant_id"
            debug "Ignoring field '#{line_split[0]}' with value '#{line_split[1]}'"
          when "updated"
            begin
              entry[:last_update]=DateTime.parse(line_split[1])
            rescue => e
              error "Failed to parse field '#{line_split[0]}' with value '#{line_split[1]}' as date", e
            end
          when "created"
            begin
              entry[:created]=DateTime.parse(line_split[1])
            rescue => e
              error "Failed to parse field '#{line_split[0]}' with value '#{line_split[1]}' as date", e
            end
          when "status"
            entry[:status]=line_split[1].downcase unless line_split[1].nil?() || line_split[1]==""
          when "progress"
            entry[:progress]=line_split[1] unless line_split[1].nil?() || line_split[1]==""
          when "name"
            entry[:name]=line_split[1] unless line_split[1].nil?() || line_split[1]==""
          when "private network"
            unless line_split[1].nil?() || line_split[1]=="" then
              entry[:private_ipv4_raw]=Array.new()
              line_split[1].strip.sub(/^private=/,'').split(',').each do |n|
                entry[:private_ipv4_raw] << n.strip
              end
            end
          when "public network"
            unless line_split[1].nil?() || line_split[1]=="" then
              entry[:public_ipv4_raw]=Array.new()
              line_split[1].strip.sub(/^private=/,'').split(',').each do |n|
                entry[:public_ipv4_raw] << n.strip
              end
            end
          when "accessIPv4"
            unless line_split[1].nil?() || line_split[1]=="" then
              entry[:access_ipv4]=Array.new()
              line.split[1].split(',').each do |n|
                entry[:access_ipv4] << @data_handler.get_ipv4_address_name(n.strip)
              end
              entry[:access_ipv4_raw]=Array.new()
              line.split[1].split(',').each do |n|
                entry[:access_ipv4_raw] << n.strip
              end
            end
          when "accessIPv6"
            warn "Skipping field '#{line_split[0]}' due to unknown data format, value '#{line_split[1]}'"
            next
#            unless line_split[1].nil?() || line_split[1]=="" then
#              entry[:access_ipv6]=Array.new()
#              line.split[1].split(',').each do |n|
#                entry[:access_ipv6] << @data_handler.get_ipv6_address_name(n.strip)
#              end
#            end
          when "image"
            entry[:image_name]=line_split[1] unless line_split[1].nil?() || line_split[1]==""
          when "id"
            entry[:id]=line_split[1] unless line_split[1].nil?() || line_split[1]==""
          when "hostId"
            entry[:host_id]=line_split[1] unless line_split[1].nil?() || line_split[1]==""
          when "flavor"
            entry[:flavor]=line_split[1] unless line_split[1].nil?() || line_split[1]==""
          when "config_drive"
            entry[:config_drive]=line_split[1] unless line_split[1].nil?() || line_split[1]==""
          else
            warn "Ignoring unknown field '#{line_split[0]}' with value '#{line_split[1]}')"
          end
        end
        if entry.include?(:id) && !entry.empty?() then
          debug "Finished detail entry for VM with id '#{entry[:id]}'"
          if @vms.include?(entry[:id].intern) then
            @vms[entry[:id].intern].merge!(entry)
          else
            @vms[entry[:id].intern]=entry
          end
        else
          error "Read vm detail entry without id; discarding data", entry.inspect unless entry.empty?()
        end
      end

      def parse_nova_create_images(label,command,result,step)
        skip=0
        entry=Hash.new()
        result[:stdout].split("\n").each do |line|
          next if is_garbage?(line)
          skip+=1
          if line.match(/^=+$/) then
            if entry.include?(:id) then
              debug "Finished detail entry for image with id '#{entry[:id]}'"
              if @images.include?(entry[:id].intern) then
                @images[entry[:id].intern].merge!(entry)
              else
                @images[entry[:id].intern]=entry
              end
            else
              error "Read image detail entry without id; discarding data", entry.inspect unless entry.empty?()
            end
            debug "Starting new detail entry"
            entry=Hash.new()
            skip=0
            next
          end
          next if skip==1
          line.strip!
          line_split=line.split(':')
          line_split.each do |field|
            field.strip!
          end
          case line_split[0]
          when "URI"
            debug "Ignoring field '#{line_split[0]}' with value '#{line_split[1]}'"
          when "Owner"
            debug "Ignoring field '#{line_split[0]}' with value '#{line_split[1]}'"
          when "Public"
            entry[:public]=(line_split[1].downcase=="yes")
          when "Protected"
            entry[:protected]=(line_split[1].downcase=="yes")
          when "Status"
            entry[:status]=line_split[1].downcase unless line_split[1].nil?() || line_split[1]==""
          when "Size"
            entry[:size]=line_split[1] unless line_split[1].nil?() || line_split[1]==""
          when "Id"
            entry[:id]=line_split[1] unless line_split[1].nil?() || line_split[1]==""
          when "Name"
            entry[:name]=line_split[1] unless line_split[1].nil?() || line_split[1]==""
          when "Disk format"
            entry[:disk_format]=line_split[1] unless line_split[1].nil?() || line_split[1]==""
          when "Container format"
            entry[:container_format]=line_split[1] unless line_split[1].nil?() || line_split[1]==""
          when "Minimum Ram Required (MB)"
            begin
              unless line_split[1].nil?() || line_split[1]==""
                entry[:minimum_ram]=Integer(line_split[1])*1024*1024
              else
                entry[:minimum_ram]=0
              end
            rescue => e
              error "Failed to parse field '#{line_split[0]}' with value '#{line_split[1]}' as integer", e
              entry[:minimum_ram]=0
            end
          when "Minimum Disk Required (GB)"
            begin
              unless line_split[1].nil?() || line_split[1]==""
                entry[:minimum_disk]=Integer(line_split[1])*1024*1024*1024
              else
                entry[:minimum_disk]=0
              end
            rescue => e
              error "Failed to parse field '#{line_split[0]}' with value '#{line_split[1]}' as integer", e
              entry[:minimum_disk]=0
            end
          else
            warn "Ignoring unknown field '#{line_split[0]}' with value '#{line_split[1]}'"
          end
        end
        if entry.include?(:id) && !entry.empty?() then
          debug "Finished detail entry for image with id '#{entry[:id]}'"
          if @images.include?(entry[:id].intern) then
            @images[entry[:id].intern].merge!(entry)
          else
            @images[entry[:id].intern]=entry
          end
        else
          error "Read image detail entry without id; discarding data", entry.inspect unless entry.empty?()
        end
        @vms.values.each do |vm|
          image_id=nil
          @images.values.each do |img|
            if img[:name]=="IO_VISA_#{vm[:id]}" && image_id.nil?() then
              image_id=img[:id]
            elsif img[:name]=="IO_VISA_#{vm[:id]}" && !image_id.nil?() && image_id!=img[:id] then
              error "Got conflicting ids '#{image_id}' and '#{img[:id]}' for image with name '#{img[:name]}'", "Skipping vm '#{vm[:id]}' with name '#{vm[:name]}'"
              image_id=nil
              break
            end
          end
          if image_id.nil?() then
            error "Got no image id for vm '#{vm[:id]}' with name '#{vm[:name]}'"
          else
            vm[:image_id]=image_id
            vm[:image]=@images[image_id.intern]
            @images[image_id.intern][:filename]="IO_VISA_#{vm[:id]}"
          end
        end
      end

      def parse_nova_config(label,command,result,step)
        result[:stdout].split("\n").each do |line|
          next if is_garbage?(line)
          next if line.match(/^#/)
          line.strip!
          line.sub!(/^--/,'')
          key,value=line.split("=")
          key.strip! unless key.nil?()
          value.strip! unless value.nil?()
          next if key.nil?() || key==""
          case key
          when "public_interface" then
            next if value.nil?() || value==""
            @config[:public_interface]=value
          when "flat_interface" then
            next if value.nil?() || value==""
            @config[:flat_interface]=value
          when "flat_network_bridge" then
            next if value.nil?() || value==""
            @config[:flat_network_bridge]=value
          when "fixed_range" then
            next if value.nil?() || value==""
            @config[:flat_network_address]=value.sub(/\/.*/,'')
            @config[:flat_network_mask]=value.sub(/[^\/]*\//,'')
            @config[:flat_network]=::IPAddress::IPv4.new(value)
          when "floating_range" then
            next if value.nil?() || value==""
            @config[:public_network_address]=value.sub(/\/.*/,'')
            @config[:public_network_mask]=value.sub(/[^\/]*\//,'')
            @config[:public_network]=::IPAddress::IPv4.new(value)
          end
        end
      end

      def parse_ovs_vs_list_br(label,command,result,step)
        result[:stdout].split("\n").each do |line|
          next if is_garbage?(line)
          @bridges[line.strip.intern]=Hash.new unless @bridges.include?(line.strip.intern)
        end
      end

      def parse_ovs_vs_show(label,command,result,step)
        last_port=nil
        result[:stdout].split("\n").each do |line|
          next if is_garbage?(line)
          l=line.strip
          next if l.match(/^Bridge/)
          if l.match(/^Port/) then
            last_port=l.sub(/[^"]*"/,'').sub(/"$/,'')
            next
          end
          @ports.values.each do |p|
            next unless p[:name]==last_port
            p[:vlans]=Array.new() unless p.include?(:vlans)
            p[:vlans] << l.sub(/tag: /,'').strip
          end if l.match(/^tag:/) && !last_port.nil?()
        end
      end

      def parse_ovs_vs_list_interface(label,command,result,step)
        data=nil
        result[:stdout].split("\n").each do |line|
          next if is_garbage?(line)
          sp_line=line.split(':')
          next if sp_line.size < 2
          key=sp_line[0].strip
          value=sp_line[1..-1].join(':').strip
          next if value==""
          if key=='_uuid'
            # starting new entry
            if @ovs_interfaces.include?(data[:uuid].intern)
              warn "Interface #{data[:uuid]} is marked as down; ignoring down state" if data[:down]
              @ovs_interfaces[data[:uuid].intern].merge!(data)
            else
              warn "#{data[:uuid]} is not associated to any port; ignoring"
            end unless data.nil?() || data.empty?() || !data.include?(:uuid)
            data=Hash.new()
            data[:uuid]=value
            next
          elsif key=='admin_state' || key=='link_state'
            data[:down]||=value=='down'
          elsif key=='name'
            data[:name]=value.gsub(/"/,'')
          elsif key=='external_ids'
            value.gsub(/[\{\}]/,'').split(',').each do |i|
              k,v=i.split('=')
              k.strip!
              v.gsub!(/"/,'')
              v.strip!
              if k=='attached-mac'
                data[:mac]=Array.new() unless data.include?(:mac)
                data[:mac] << v
              elsif k=='iface-id'
                data[:id]=v
              elsif k=='vm-uuid'
                data[:vm]=v
              end
            end
          end
        end
        if @ovs_interfaces.include?(data[:uuid].intern)
          warn "Interface #{data[:uuid]} is marked as invalid; ignoring" if data[:invalid]
          @ovs_interfaces[data[:uuid].intern].merge!(data) unless data[:invalid]
        else
          warn "#{data[:uuid]} is not associated to any port; ignoring"
        end unless data.nil?() || data.empty?() || !data.include?(:uuid)
      end

      def parse_ovs_vs_list_bridge(label,command,result,step)
        data=nil
        result[:stdout].split("\n").each do |line|
          next if is_garbage?(line)
          sp_line=line.split(':')
          next if sp_line.size < 2
          key=sp_line[0].strip
          value=sp_line[1..-1].join(':').strip
          next if value==""
          if key=='_uuid'
            # starting new entry
            unless data.nil?() || !data.include?(:name)
              @bridges[data[:name].intern]=Hash.new() unless @bridges.include?(data[:name].intern)
              @bridges[data[:name].intern].merge!(data)
            end
            data=Hash.new()
            data[:uuid]=value
            next
          elsif key=='name'
            data[:name]=value
          elsif key=='ports'
            data[:ports]=Array.new()
            value.gsub(/[\[\]]/,'').split(',').each do |p|
              data[:ports] << p.strip
              @ports[p.strip.intern]=Hash.new unless @ports.include?(p.strip.intern)
            end
          end
        end
        unless data.nil?() || data.empty?()
          @bridges[data[:name].intern]=Hash.new() unless @bridges.include?(data[:name].intern)
          @bridges[data[:name].intern].merge!(data)
        end
      end

      def parse_ovs_vs_list_port(label,command,result,step)
        data=nil
        r=result[:stdout].split("\n")
        result[:stdout].split("\n").each do |line|
          next if is_garbage?(line)
          sp_line=line.split(':')
          next if sp_line.size < 2
          key=sp_line[0].strip
          value=sp_line[1..-1].join(':').strip
          next if value==""
          if key=='_uuid'
            # starting new entry
            if @ports.include?(data[:uuid].intern)
              @ports[data[:uuid].intern].merge!(data)
            else
              warn "#{data[:uuid]} is not associated to any bridge; ignoring"
            end unless data.nil?() || data.empty?() || !data.include?(:uuid)
            data=Hash.new()
            data[:uuid]=value
            next
          elsif key=='name'
            data[:name]=value.gsub(/"/,'')
          elsif key=='interfaces'
            data[:interfaces]=Array.new()
            value.gsub(/[\[\]]/,'').split(',').each do |i|
              data[:interfaces] << i.strip
              @ovs_interfaces[i.strip.intern]=Hash.new unless @ovs_interfaces.include?(i.strip.intern)
            end
          end
        end
        if @ports.include?(data[:uuid].intern)
          @ports[data[:uuid].intern].merge!(data)
        else
          warn "#{data[:uuid]} is not associated to any bridge; ignoring"
        end unless data.nil?() || data.empty? || !data.include?(:uuid)
      end

      #fake switches in VSA
      def postprocess_parsed_data_plain()
        vsa=@data_handler.get_group(:vsa,@host[:vsa])
        vsa[:no_network_structure]=@host[:network_structure]

        public_master_switch=nil
        public_bridge=nil
        private_master_switch=nil
        private_bridge=nil
        connections=Array.new()
        endpoints=Array.new()
        host=Iof::Data::NO::ManagedNetworkComponent.new()
        host[:no_vsa_host]=true
        host.name=@host[:name]

        if vsa[:no_network_structure]==:plain then
          public_master_switch=Iof::Data::NO::UnmanagedNetworkComponent.new()
          public_master_switch.name="openstack_plain_#{@host[:vsa]}_public_master"
          public_master_switch[:no_vsa_infrastructure]=true
          public_l1_from_master=Iof::Data::NO::Layer1Interface.new()
          public_l1_from_master.name="l1_public_#{@host[:name]}"
          public_master_switch[:no_layer1_interface]=public_l1_from_master

          public_bridge=Iof::Data::NO::UnmanagedNetworkComponent.new()
          public_bridge.name="openstack_plain_#{@host[:vsa]}_#{@host[:name]}_#{@config[:public_interface]}"
          public_bridge[:no_vsa_infrastructure]=true
          public_l1_to_master=Iof::Data::NO::Layer1Interface.new()
          public_l1_to_master.name="l1_public_master"
          public_bridge[:no_layer1_interface]=public_l1_to_master
          public_bridge[:no_runs_on]=host.name

          public_l1_connection=Iof::Data::NO::IsConnectedTo.new()
          public_l1_connection.subject="#{public_master_switch.name}___#{public_l1_from_master.name}"
          public_l1_connection.object="#{public_bridge.name}___#{public_l1_to_master.name}"
          connections << public_l1_connection

          private_master_switch=Iof::Data::NO::UnmanagedNetworkComponent.new()
          private_master_switch.name="openstack_plain_#{@host[:vsa]}_private_master"
          private_master_switch[:no_vsa_infrastructure]=true
          private_l1_from_master=Iof::Data::NO::Layer1Interface.new()
          private_l1_from_master.name="l1_private_#{@host[:name]}"
          private_master_switch[:no_layer1_interface]=private_l1_from_master

          private_bridge=Iof::Data::NO::UnmanagedNetworkComponent.new()
          private_bridge.name="openstack_plain_#{@host[:vsa]}_#{@host[:name]}_#{@config[:flat_network_bridge]}"
          private_bridge[:no_vsa_infrastructure]=true
          private_l1_to_master=Iof::Data::NO::Layer1Interface.new()
          private_l1_to_master.name="l1_private_master"
          private_bridge[:no_layer1_interface]=private_l1_to_master
          private_bridge[:no_runs_on]=host.name

          private_l1_connection=Iof::Data::NO::IsConnectedTo.new()
          private_l1_connection.subject="#{private_master_switch.name}___#{private_l1_from_master.name}"
          private_l1_connection.object="#{private_bridge.name}___#{private_l1_to_master.name}"
          connections << private_l1_connection
        elsif vsa[:no_network_structure]==:openvswitch then
          error "Not supported yet"
        else
          warn "Not creating any connections"
        end

        hosts=Array.new()

        @vms.each do |id,data|
          unless data[:image].nil?() || data[:image].empty?()
            hosts << data
          else
            error "VM with id '#{id.to_s}' and name '#{data[:name]}' has no associated image"
          end
        end

        hosts.each do |h|
          vm=Iof::Data::NO::ManagedNetworkComponent.new()
          endpoints << vm
          vm.name=h[:name]
          vm[:no_vsa_origin_id]=h[:id]
          vm[:no_image_file]=h[:image][:filename] if h.include?(:image) && h[:image].include?(:filename)
          vm[:no_runs_on]=host.name
          with_public_interface=false
          public_addresses=Array.new()
          with_private_interface=false
          private_addresses=Array.new()
          with_access_interface=!((h[:access_ipv4].nil?() || h[:access_ipv4].empty?)&&(h[:access_ipv6].nil?() || h[:access_ipv6].empty?))
          h[:private_ipv4_raw].each do |i|
            if @config[:public_network].include?(::IPAddress::IPv4.new(i)) then
              with_public_interface=true
              public_addresses << @data_handler.get_ipv4_address_name(i,@config[:public_network_mask])
            end unless @config[:public_network].nil?()
            if @config[:flat_network].include?(::IPAddress::IPv4.new(i)) then
              with_private_interface=true
              private_addresses << @data_handler.get_ipv4_address_name(i,@config[:flat_network_mask])
            end unless @config[:flat_network].nil?()
          end unless h[:private_ipv4_raw].nil?()
          if with_private_interface then
            l1_flat=Iof::Data::NO::Layer1Interface.new()
            l1_flat.name="l1_flat"
            l2_flat=Iof::Data::NO::EthernetInterface.new()
            l2_flat.name="l2_flat"
            l1_flat[:no_ethernet_interface]=l2_flat
            count=0
            private_addresses.each do |a|
              l3_flat=Iof::Data::NO::IPv4Interface.new()
              l3_flat.name="l3_flat_#{count}"
              count+=1
              l3_flat[:no_ipv4_address]=a
              l2_flat[:no_ipv4_interface]=l3_flat
            end unless private_addresses.nil?()
            vm[:no_layer1_interface]=l1_flat

            l1_flat_bridge=Iof::Data::NO::Layer1Interface.new()
            l1_flat_bridge.name="l1_#{vm.name}"
            private_bridge[:no_layer1_interface]=l1_flat_bridge

            c=Iof::Data::NO::IsConnectedTo.new()
            c.subject="#{vm.name}___#{l1_flat.name}"
            c.object="#{private_bridge.name}___#{l1_flat_bridge.name}"
            connections << c
          end

          if with_public_interface then
            l1_public=Iof::Data::NO::Layer1Interface.new()
            l1_public.name="l1_public"
            l2_public=Iof::Data::NO::EthernetInterface.new()
            l2_public.name="l2_public"
            l1_public[:no_ethernet_interface]=l2_public
            count=0
            public_addresses.each do |a|
              l3_public=Iof::Data::NO::IPv4Interface.new()
              l3_public.name="l3_public_#{count}"
              count+=1
              l3_public[:no_ipv4_address]=a
              l2_public[:no_ipv4_interface]=l3_public
            end unless public_addresses.nil?()
            h[:access_ipv4].each do |a|
              l3_public=Iof::Data::NO::IPv4Interface.new()
              l3_public.name="l3_public_#{count}"
              count+=1
              l3_public[:no_ipv4_address]=a
              l2_public[:no_ipv4_interface]=l3_public
            end if with_access_interface && !h[:access_ipv4].nil?()
            count=0
            h[:access_ipv6].each do |a|
              l3_public=Iof::Data::NO::IPv6Interface.new()
              l3_public.name="l3_public_#{count}"
              count+=1
              l3_public[:no_ipv6_address]=a
              l2_public[:no_ipv6_interface]=l3_public
            end if with_access_interface && !h[:access_ipv6].nil?()
            vm[:no_layer1_interface]=l1_public

            l1_public_bridge=Iof::Data::NO::Layer1Interface.new()
            l1_public_bridge.name="l1_#{vm.name}"
            public_bridge[:no_layer1_interface]=l1_public_bridge

            c=Iof::Data::NO::IsConnectedTo.new()
            c.subject="#{vm.name}___#{l1_public.name}"
            c.object="#{public_bridge.name}___#{l1_public_bridge.name}"
            connections << c
          end
        end

        @results[:master_switches]=[public_master_switch,private_master_switch]
        @results[:bridges]=[public_bridge,private_bridge]
        @results[:host]=host
        @results[:endpoints]=endpoints
        @results[:connections]=connections
        @results[:vsa]=@host[:vsa]


        @vms.clear
        @images.clear
        @interfaces.clear
        @host.clear
        @config.clear
      end

      def postprocess_parsed_data_openvswitch()
        vsa=@data_handler.get_group(:vsa,@host[:vsa])
        vsa[:no_network_structure]=@host[:network_structure]

        bridges=Hash.new()
        interfaces=Hash.new()
        vm_to_interface=Hash.new()
        networks=Hash.new()
        interface_to_vlan=Hash.new()

        host=Iof::Data::NO::ManagedNetworkComponent.new()
        host[:no_vsa_host]=true
        host.name=@host[:name]


        unknown_bridge=Iof::Data::NO::UnmanagedNetworkComponent.new()
        unknown_bridge.name="#{host.name}_unknown"
        unknown_bridge[:no_runs_on]=host.name
        unknown=false


        @bridges.values.each do |bridge|
          b=Iof::Data::NO::Switch.new()
          b.name="#{host.name}_#{bridge[:name]}"
          b[:no_runs_on]=host.name
          gw=nil
          bridge[:ports].each do |port|
            real_port=@ports[port.intern]
            next if real_port.nil?()
            if real_port[:name].match(/^tap/) then
              #connected to VM
              l2=Iof::Data::NO::EthernetInterface.new()
              l2.name="l2_eth_#{real_port[:name]}"
              l1=Iof::Data::NO::Layer1Interface.new()
              l1.name=real_port[:name]
              l1[:no_ethernet_interface]=l2
              real_port[:interfaces].each do |interface|
                i=@ovs_interfaces[interface.intern]
                if real_port[:vlans].size==1
                  l2[:no_native_vlan]=@data_handler.get_vlan_name(real_port[:vlans][0])
                  interface_to_vlan[interface.intern]=Array.new() unless interface_to_vlan.include?(interface.intern)
                  interface_to_vlan[interface.intern] << real_port[:vlans][0]
                else
                  real_port[:vlans].each do |v|
                    l2[:no_tagged_vlan]=@data_handler.get_vlan_name(v)
                    interface_to_vlan[interface.intern]=Array.new() unless interface_to_vlan.include?(interface.intern)
                    interface_to_vlan[interface.intern] << v
                  end
                end if real_port.include?(:vlans) && !real_port[:vlans].empty?()
                next if i.nil?()
                unless i[:vm].nil?()
                  vm_to_interface[i[:vm].intern]=Array.new() unless vm_to_interface.include?(i[:vm].intern)
                  vm_to_interface[i[:vm].intern] << interface.intern
                  interfaces[interface.intern]="#{b.name}___#{l1.name}"
                end
                i[:mac].each do |m|
                  l2[:no_mac_address]=@data_handler.get_mac_address_name(m)
                end unless i[:mac].nil?()
              end if real_port.include?(:interfaces)
              b[:no_layer1_interface]=l1
            elsif real_port[:name].match(/^gw/) then
              #used as "router" interface
              if gw.nil?() then
                gw=Iof::Data::NO::Layer1Interface.new()
                gw.name="l1_virt_#{b.name}_gw"
              end
              l2=Iof::Data::NO::EthernetInterface.new()
              l2.name="l2_eth_#{real_port[:name]}"
              l3v4=Iof::Data::NO::IPv4Interface.new()
              l3v4.name="l3_ipv4_#{real_port[:name]}"
              gw[:no_ethernet_interface]=l2
              real_port[:interfaces].each do |interface|
                interfaces[interface.intern]="#{b.name}___#{gw.name}"
                i=@ovs_interfaces[interface.intern]
                next if i.nil?()
                if real_port[:vlans].size==1
                  l2[:no_native_vlan]=@data_handler.get_vlan_name(real_port[:vlans][0])
                  interface_to_vlan[interface.intern]=Array.new() unless interface_to_vlan.include?(interface.intern)
                  interface_to_vlan[interface.intern] << real_port[:vlans][0]
                else
                  real_port[:vlans].each do |v|
                    l2[:no_tagged_vlan]=@data_handler.get_vlan_name(v)
                    interface_to_vlan[interface.intern]=Array.new() unless interface_to_vlan.include?(interface.intern)
                    interface_to_vlan[interface.intern] << v
                  end
                end if real_port.include?(:vlans) && !real_port[:vlans].empty?()
                 unless i[:vm].nil?()
                  vm_to_interface[i[:vm].intern]=Array.new() unless vm_to_interface.include?(i[:vm].intern)
                  vm_to_interface[i[:vm].intern] << interface.intern
                end
                i[:mac].each do |m|
                  l2[:no_mac_address]=@data_handler.get_mac_address_name(m)
                end unless i[:mac].nil?()
              end
              if @interfaces.include?(real_port[:name].intern) && @interfaces[real_port[:name].intern].include?(:ipv4) then
                l3v4=Iof::Data::NO::IPv4Interface.new()
                l3v4.name="l3_ipv4_#{real_port[:name]}"
                @interfaces[real_port[:name].intern][:ipv4][:addr].each do |a|
                  l3v4[:no_ipv4_address]=a
                end
                @interfaces[real_port[:name].intern][:ipv4][:net].each do |n|
                  l3v4[:no_ipv4_network]=n
                end
                real_port[:vlans].each do |v|
                  networks[v.intern]=Array.new unless networks.include?(v.intern)
                  networks[v.intern].concat(@interfaces[real_port[:name].intern][:ipv4][:net_raw])
                end unless real_port[:vlans].nil?()
              end
              #v6 is disabled
              if false && @interfaces.include?(real_port[:name].intern) && @interfaces[real_port[:name].intern].include?(:ipv6) then
                l3v6=Iof::Data::NO::IPv6Interface.new()
                l3v6.name="l3_ipv6_#{real_port[:name]}"
                @interfaces[real_port[:name].intern][:ipv6][:addr].each do |a|
                  l3v6[:no_ipv6_address]=a
                end
                @interfaces[real_port[:name].intern][:ipv6][:net].each do |n|
                  l3v6[:no_ipv6_address]=n
                end
                real_port[:vlans].each do |v|
                  networks[v.intern]=Array.new unless networks.include?(v.intern)
                  networks[v.intern].concat(@interfaces[real_port[:name].intern][:ipv4][:net_raw])
                end unless real_port[:vlans].nil?()
              end
            else
              #ignore real interfaces
            end
          end
          bridges[bridge[:uuid].intern]=b
        end

        connections=Array.new()
        endpoints=Array.new()
        host=Iof::Data::NO::ManagedNetworkComponent.new()
        host[:no_vsa_host]=true
        host.name=@host[:name]

        hosts=Array.new()

        @vms.each do |id,data|
          error "VM with id '#{id.to_s}' and name '#{data[:name]}' has no associated image" if data[:image].nil?() || data[:image].empty?
          hosts << data
        end

        hosts.each do |h|
          vm=Iof::Data::NO::ManagedNetworkComponent.new()
          endpoints << vm
          vm.name=h[:name]
          vm[:no_vsa_origin_id]=h[:id]
          vm[:no_image_file]=h[:image][:filename] if h.include?(:image) && h[:image].include?(:filename)
          vm[:no_runs_on]=host.name

          all_nets=Array.new()
          ifaces=vm_to_interface[h[:id].intern]
          ifaces.each do |i|
            vlans=interface_to_vlan[i.intern]
            nets=Array.new
            addr=Array.new()
            vlans.each do |v|
              nets.concat(networks[v.intern])
              all_nets.concat(networks[v.intern])
            end unless vlans.nil?()
            h[:private_ipv4_raw].each do |a|
              nets.each do |n|
                addr << [a,n] if n.include?(::IPAddress::IPv4.new(a))
              end
            end unless h[:private_ipv4_raw].nil?()
            h[:public_ipv4_raw].each do |a|
              nets.each do |n|
                addr << [a,n] if n.include?(::IPAddress::IPv4.new(a))
              end
            end unless h[:public_ipv4_raw].nil?()
            h[:access_ipv4_raw].each do |a|
              nets.each do |n|
                addr << [a,n] if n.include?(::IPAddress::IPv4.new(a))
              end
            end unless h[:access_ipv4_raw].nil?()
            l1=Iof::Data::NO::Layer1Interface.new()
            l1.name=@ovs_interfaces[i.intern][:name]
            vm[:no_layer1_interface]=l1
            l2=Iof::Data::NO::EthernetInterface.new()
            l2.name="l2_eth_#{l1.name}"
            l1[:no_ethernet_interface]=l2
            l3=Iof::Data::NO::IPv4Interface.new()
            l3.name="l3_ipv4_#{l1.name}"
            l2[:no_ipv4_interface]=l3
            addr.each do |a|
              if !a[1].nil?() && !a[0].nil?()
                l3[:no_ipv4_address]=@data_handler.get_ipv4_address_name(a[0],a[1].prefix.to_s)
                l3[:no_ipv4_network]=@data_handler.get_ipv4_network_name(a[1].network.to_s,a[1].prefix.to_s)
              elsif a[1].nil?() && !a[0].nil?()
                l3[:no_ipv4_address]=@data_handler.get_ipv4_address_name(a[0])
              end
            end
            c=Iof::Data::NO::IsConnectedTo.new()
            c.subject="#{vm.name}___#{l1.name}"
            c.object=interfaces[i.intern]
            connections << c
          end unless ifaces.nil?()

          open_addresses=Array.new()
          h[:private_ipv4_raw].each do |a|
            match=false
            all_nets.each do |n|
              match ||= n.include?(::IPAddress::IPv4.new(a))
            end
            open_addresses << a unless match
          end unless h[:private_ipv4_raw].nil?()
          h[:public_ipv4_raw].each do |a|
            match=false
            all_nets.each do |n|
              match ||= n.include?(::IPAddress::IPv4.new(a))
            end
            open_addresses << a unless match
          end unless h[:public_ipv4_raw].nil?()
          h[:access_ipv4_raw].each do |a|
            match=false
            all_nets.each do |n|
              match ||= n.include?(::IPAddress::IPv4.new(a))
            end
            open_addresses << a unless match
          end unless h[:access_ipv4_raw].nil?()

          unless open_addresses.nil?() || open_addresses.empty?()
            l1=Iof::Data::NO::Layer1Interface.new()
            l1.name="l1_unknown"
            vm[:no_layer1_interface]=l1
            l2=Iof::Data::NO::EthernetInterface.new()
            l2.name="l2_eth_#{l1.name}"
            l1[:no_ethernet_interface]=l2
            l3=Iof::Data::NO::IPv4Interface.new()
            l3.name="l3_ipv4_#{l1.name}"
            l2[:no_ipv4_interface]=l3
            open_addresses.each do |a|
              l3[:no_ipv4_address]=@data_handler.get_ipv4_address_name(a) unless a.nil?()
            end

            l1_bridge=Iof::Data::NO::Layer1Interface.new()
            l1_bridge.name="l1_#{vm.name}"
            unknown_bridge[:no_layer1_interface]=l1_bridge

            c=Iof::Data::NO::IsConnectedTo.new()
            c.subject="#{vm.name}___#{l1.name}"
            c.object="#{unknown_bridge.name}___#{l1_bridge.name}"
            connections << c
            unknown=true
          end
        end

        @results[:master_switches]=[]
        @results[:bridges]=bridges.values
        @results[:bridges] << unknown_bridge if unknown
        @results[:host]=host
        @results[:endpoints]=endpoints
        @results[:connections]=connections
        @results[:vsa]=@host[:vsa]

        @vms.clear
        @images.clear
        @interfaces.clear
        @host.clear
        @config.clear
      end


      def send_results()
        @data_handler.set_component(@results[:host])
        @results[:endpoints].each do |e|
          @data_handler.set_component(e)
          @data_handler.set_group_member(:vsa,@results[:vsa],e.name)
        end
        @results[:bridges].each do |b|
          @data_handler.set_component(b)
          @data_handler.set_group_member(:vsa,@results[:vsa],b.name)
        end
        @results[:master_switches].each do |s|
          @data_handler.set_neighbor(s)
          @data_handler.set_group_member(:vsa,@results[:vsa],s.name)
        end
        @results[:connections].each do |c|
          @data_handler.set_connection(c)
        end
      end
    end
  end
end
