# encoding: utf-8
# license: gpl3p

### BEGIN LICENSE NOTICE
# This file is part of the interconnect asset ontology framework (IO)
# Copyright (C) 2010 - 2013
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE NOTICE

### BEGIN AUTHOR LIST
#
### END AUTHOR LIST
module Iof
  module Parser
    class VisaRDF < Iof::Parser::Base

      def initialize(data_handler)
        super(nil,data_handler,nil)
        raise if @data_handler.nil?()
        @ontology=@data_handler.ontology
      end

      def is_managed_switch?(d)
        unmanaged=true
        d[:interfaces].each do |i|
          unmanaged&&=(i[:vlans].nil?() || i[:vlans].empty?())
          unmanaged&&=(i[:addresses].nil?() || i[:addresses].empty?())
        end
        return !unmanaged
      end

      def write(import)
        import.each do |v|
          vsa=@data_handler.get_group_name(:vsa, v.name)
          debug "Creating vsa group with name #{vsa}"
          todo_connections=Hash.new()
          done_connections=Array.new()
          v.devices.each do |d|
            klass=case d[:type]
            when:switch
              #and here the magic appears
              #UnmanagedSwitch for switch without config
              #in other cases Switch
              if is_managed_switch?(d)
                Iof::Data::NO::Switch
              else
                Iof::Data::NO::UnmanagedNetworkComponent
              end
            when :host
              Iof::Data::NO::ManagedNetworkComponent
            else
              nil
            end
            if klass.nil?()
              error "Could not determine class for device #{d.inspect}"
              next
            end
            device=klass.new()

            if d[:internal_name].nil?()
              if d[:uri].nil?()
                device.name="#{d[:name]}#{(d[:suffix]!='')?("_#{d[:suffix]}"):("")}"
              else
                device.name="#{d[:uri]}#{(d[:suffix]!='')?("_#{d[:suffix]}"):("")}"
              end
            else
              device.name="#{d[:internal_name]}#{(d[:suffix]!='')?("_#{d[:suffix]}"):("")}"
            end
            if d[:name].nil?()
              if d[:internal_name].nil?()
                device[:no_topology_editor_name]=d[:uri]
              else
                device[:no_topology_editor_name]=d[:internal_name]
              end
            else
              device[:no_topology_editor_name]=d[:name]
            end
            device[:no_image_file]=d[:image_file]
            device[:no_topology_editor_group]=d[:group][:name] unless d[:group].nil?()
            d[:interfaces].each do |i|
              l1=Iof::Data::NO::Layer1Interface.new()
              if i[:internal_name].nil?()
                l1.name=i[:name].sub(/_[a-z0-9A-Z]{32}$/,'').sub(/^.*___/,'') unless i[:name].nil?()
              else
                l1.name=i[:internal_name].sub(/_[a-z0-9A-Z]{32}$/,'').sub(/^.*___/,'')
              end
              device[:no_layer1_interface]=l1
              l2=nil
              unless (i[:vlans].nil?() || i[:vlans].empty?()) && (i[:addresses].nil?() || i[:addresses].empty?())
                l2=Iof::Data::NO::EthernetInterface.new()
                l2.name="l2_eth_#{l1.name}"
                if i[:vlans].size==1
                  l2[:no_native_vlan]=@data_handler.get_vlan_name(i[:vlans][0][:id])
                else
                  i[:vlans].each do |vlan|
                    l2[:no_tagged_vlan]=@data_handler.get_vlan_name(vlan[:id])
                  end
                end unless (i[:vlans].nil?() || i[:vlans].empty?())
                l1[:no_ethernet_interface]=l2
                unless (i[:addresses].nil?() || i[:addresses].empty?())
                  l3_v4=Iof::Data::NO::IPv4Interface.new()
                  l3_v4.name="l3_ipv4_#{l1.name}"
                  l3_v6=Iof::Data::NO::IPv6Interface.new()
                  l3_v6.name="l3_ipv6_#{l1.name}"
                  v4=false
                  v6=false
                  i[:addresses].each do |a|
                    case a[:type]
                    when :ethernet
                      l2[:no_mac_address]=@data_handler.get_mac_address_name(a[:value])
                    when :ipv4
                      v4=true
                      if a[:network].nil?()
                        l3_v4[:no_ipv4_address]=@data_handler.get_ipv4_address_name(a[:value])
                      else
                        l3_v4[:no_ipv4_address]=@data_handler.get_ipv4_address_name(a[:value],a[:network][:netmask_length])
                        l3_v4[:no_ipv4_network]=@data_handler.get_ipv4_network_name(a[:network][:value],a[:network][:netmask_length])
                      end
                    when :ipv6
                      v6=true
                      if a[:network].nil?()
                        l3_v6[:no_ipv6_address]=@data_handler.get_ipv6_address_name(a[:value])
                      else
                        l3_v6[:no_ipv6_address]=@data_handler.get_ipv6_address_name(a[:value],a[:network][:netmask_length])
                        l3_v6[:no_ipv6_network]=@data_handler.get_ipv6_network_name(a[:network][:value],a[:network][:netmask_length])
                      end
                    else
                      warn "Unknown address type #{a[:type]} on interface #{i[:internal_name]}"
                    end

                  end
                  l2[:no_ipv4_interface]=l3_v4 if v4
                  l2[:no_ipv6_interface]=l3_v6 if v6
                else
                  debug "No addresses for #{i.inspect}"
                end
              else
                debug "No vlans and addresses for #{i.inspect}"
              end
              if todo_connections.include?("#{device.name}___#{l1.name}".intern)
                todo=todo_connections["#{device.name}___#{l1.name}".intern]
                s=nil
                s="#{device.name}___#{l1.name}"
                o=nil
                o="#{todo[:device].name}___#{todo[:l1].name}"
                if s.nil?() || o.nil?()
                  error "Failed to create names for connection between #{i[:internal_name]} and #{todo[:id]}"
                else
                  c1=Iof::Data::NO::IsConnectedTo.new()
                  c2=Iof::Data::NO::IsConnectedTo.new()
                  c1.subject=s.clone
                  c1.object=o.clone
                  c2.subject=o.clone
                  c2.object=s.clone
                  done_connections << c1
                  done_connections << c2
                end
                todo_connections.delete("#{device.name}___#{l1.name}".intern)
              elsif todo_connections.include?(l1.name.intern)
                todo=todo_connections[l1.name.intern]
                s=nil
                s="#{device.name}___#{l1.name}"
                o=nil
                o="#{todo[:device].name}___#{todo[:l1].name}"
                if s.nil?() || o.nil?()
                  error "Failed to create names for connection between #{i[:internal_name]} and #{todo[:id]}"
                else
                  c1=Iof::Data::NO::IsConnectedTo.new()
                  c2=Iof::Data::NO::IsConnectedTo.new()
                  c1.subject=s.clone
                  c1.object=o.clone
                  c2.subject=o.clone
                  c2.object=s.clone
                  done_connections << c1
                  done_connections << c2
                end
                todo_connections.delete(l1.name.intern)
              else
                todo_connections[i[:connected].intern]={ :device => device, :l1 => l1 , :id => "#{device.name}___#{l1.name}".intern}
              end unless i[:connected].nil?()
            end
            @data_handler.set_component(device)
            @data_handler.set_group_member(:vsa, v.name, device.name)
          end unless v.devices.nil?()
          todo_connections.each do |id,c|
            error "Failed to create connection between #{id.to_s} and #{c[:id]}"
          end
          done_connections.each do |c|
            @data_handler.set_connection(c)
          end
        end unless import.nil?()
      end

      def update(import)
        #update mapped write_new
        #in current state of VISA no relevant data are hold in the ontology only
        write(import)
      end
    end
  end
end
