# encoding: utf-8
# license: gpl3p

### BEGIN LICENSE NOTICE
# This file is part of the interconnect asset ontology framework (IO)
# Copyright (C) 2010 - 2013
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE NOTICE

### BEGIN AUTHOR LIST
#
### END AUTHOR LIST
require 'rdf/raptor'
module Iof
  module Query
    class Openstack < Base
      def initialize(ontology_file)
        super(:openstack,[],ontology_file)
      end

      def query_topology()
        vsas_raw=@ontology.query("SELECT DISTINCT ?v WHERE { ?v a no:VSA}")
        vsas=Hash.new()
        vsas_raw.each do |v|
          vsa_name=v[:v].sub(/__vsa_(.*)__/,"\\1")
          next if vsa_name.nil?() || vsa_name==""
          vsas[v[:v].intern]=VSA.new(vsa_name,v[:v],Hash.new())
        end unless vsas_raw.nil?()
        vsas.values.each do |v|
          members=@ontology.query("SELECT DISTINCT ?c ?m WHERE { no:#{v[:internal_name]} no:hasVSAMember ?m . ?m a ?c }",true)
          member_types=Hash.new()
          members.each do |m|
            if m[:c]=="Switch" || m[:c]=="UnmanagedNetworkComponent"
              member_types[m[:m].intern]=:switch
            elsif m[:c]=="ManagedNetworkComponent" && !member_types.include?(m[:m].intern)
              member_types[m[:m].intern]=:host
            end
          end
          members.each do |m|
            next if m[:c]=="NetworkComponent"
            type=:none
            type=member_types[m[:m].intern] if member_types.include?(m[:m].intern)
            device=Device.new(m[:m],m[:m],type,nil,false,Hash.new())
            name=@ontology.query("SELECT DISTINCT ?n WHERE { no:#{m[:m]} no:topologyEditorName ?n }",true)
            device[:name]=name[0][:n] unless name.nil?() || name.empty?() || name[0].nil?()
            image=@ontology.query("SELECT DISTINCT ?i WHERE { no:#{m[:m]} no:imageFile ?i }",true)
            device[:image_file]=image[0][:i] unless image.nil?() || image.empty?() || image[0].nil?()
            group=@ontology.query("SELECT DISTINCT ?g WHERE { no:#{m[:m]} no:topologyEditorGroup ?g }",true)
            device[:group]=Group.new(Iof::Data::Base.sanitize_name(group[0][:g]).sub(/\//,'_'),group[0][:g]) unless group.nil?() || group.empty?() || group[0].nil?()
            v[:devices][m[:m].intern]=device
          end
        end
        vsas.values.each do |v|
          v[:devices].values.each do |d|
            interfaces_raw=@ontology.query("SELECT DISTINCT ?i WHERE { no:#{d[:internal_name]} no:hasLayer1Interface ?i }",true)
            interfaces=d[:interfaces]
            interfaces_raw.each do |i|
              name=i[:i].clone()
              name.sub!(/flat/,'private')
              interface=Interface.new(name,i[:i],Array.new(),Array.new(),nil)
              interface[:internal_identifier]=interface[:name].sub(/^.*?___/,'')
              interfaces[i[:i].intern]=interface
            end unless interfaces_raw.nil?()
            interfaces.values.each do |i|
              macs=@ontology.query("SELECT DISTINCT ?mac ?m WHERE { no:#{i[:internal_name]} no:hasEthernetInterface ?l2 . ?l2 no:hasMacAddress ?mac . ?mac no:hasMacAddressValue ?m }",true)
              macs.each do |m|
                i[:addresses] << Address.new(:ethernet,m[:m],nil,m[:mac])
              end unless macs.nil?()
              ipv4=@ontology.query("SELECT DISTINCT ?ipv4 ?a ?l WHERE { no:#{i[:internal_name]} no:hasEthernetInterface ?l2 . ?l2 no:hasIPv4Interface ?l3 . ?l3 no:hasIPv4Address ?ipv4 . ?ipv4 no:hasIPv4AddressValue ?a . OPTIONAL { ?ipv4 no:hasIPv4NetmaskLength ?l }}",true)
              ipv4.each do |a|
                if a.include?(:l) then
                  n=Network.new(:ipv4,Iof::Data.ipv4_network_part(a[:a],a[:l]),a[:l],"__ipv4_network_#{Iof::Data.ipv4_network_part(a[:a],a[:l])}/#{a[:l]}__")
                  i[:addresses] << Address.new(:ipv4,"#{a[:a]}",n,a[:ipv4])
                else
                  i[:addresses] << Address.new(:ipv4,a[:a],nil,a[:ipv4])
                end
              end unless ipv4.nil?()
              ipv6=@ontology.query("SELECT DISTINCT ?ipv6 ?a ?l WHERE { no:#{i[:internal_name]} no:hasEthernetInterface ?l2 . ?l2 no:hasIPv6Interface ?l3 . ?l3 no:hasIPv6Address ?ipv6 . ?ipv6 no:hasIPv6AddressValue ?a . OPTIONAL { ?ipv6 no:hasIPv6NetmaskLength ?l }}",true)
              ipv6.each do |a|
                if a.include?(:l) then
                  n=Network.new(:ipv6,Iof::Data.ipv6_network_part(a[:a],a[:l]),a[:l],"__ipv6_network_#{Iof::Data.ipv6_network_part(a[:a],a[:l])}/#{a[:l]}__")
                  i[:addresses] << Address.new(:ipv6,"#{a[:a]}",n,a[:ipv6])
                else
                  i[:addresses] << Address.new(:ipv6,a[:a],a[:ipv6])
                end
              end unless ipv6.nil?()
              vlans=@ontology.query("SELECT DISTINCT ?vlan ?v WHERE { no:#{i[:internal_name]} no:hasEthernetInterface ?l2 . ?l2 no:hasVlan ?vlan . ?vlan no:hasVlanID ?v }",true)
              vlans.each do |vlan|
                i[:vlans] << Vlan.new(vlan[:vlan],vlan[:v])
              end unless vlans.nil?()
            end unless interfaces.nil?()
          end unless v[:devices].nil?()
        end unless vsas.nil?()
        connections_raw=@ontology.query("SELECT ?a ?b ?a_l1 ?b_l1 WHERE { ?a no:hasLayer1Connection ?b . ?a no:hasLayer1Interface ?a_l1 . ?b no:hasLayer1Interface ?b_l1 . ?a_l1 no:isConnectedTo ?b_l1 }")
        connections_raw.each do |c|
          vsa_a=nil
          vsa_b=nil
          a=nil
          b=nil
          vsas.values.each do |v|
            if v[:devices].include?(c[:a].intern) && v[:devices].include?(c[:b].intern) then
              #both devices of this connection belong to the current vsa
              vsa_a=v
              vsa_b=v
              a=v[:devices][c[:a].intern]
              b=v[:devices][c[:b].intern]
            elsif v[:devices].include?(c[:a].intern) && !v[:devices].include?(c[:b].intern) then
              #the first device of this connection belongs to the current vsa
              vsa_a=v
              a=v[:devices][c[:a].intern]
            elsif !v[:devices].include?(c[:a].intern) && v[:devices].include?(c[:b].intern) then
              #the second devices of this connection belongs to the current vsa
              vsa_b=v
              b=v[:devices][c[:b].intern]
            else
              #no device of this connection belongs to the current vsa
            end
            break unless vsa_a.nil?() || vsa_b.nil?()
          end unless vsas.nil?()
          if vsa_a.nil?() || vsa_b.nil?() then
            warn "Found no vsa at least end of the connection; skipping connection"
          elsif a.nil?() || b.nil?() then
            warn "Got empty devices; skipping connection"
          else
            i_a=a[:interfaces][c[:a_l1].intern]
            i_b=b[:interfaces][c[:b_l1].intern]
            if i_a.nil?() || i_b.nil?() then
              warn "Got empty interfaces; skipping connection"
            else
              i_a[:connected]=i_b[:internal_name]
              i_b[:connected]=i_a[:internal_name]
            end
          end
        end unless connections_raw.nil?()
        return vsas
      end
    end
  end
end
