# encoding: utf-8
# license: gpl3p

### BEGIN LICENSE NOTICE
# This file is part of the interconnect asset ontology framework (IO)
# Copyright (C) 2010 - 2013
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE NOTICE

### BEGIN AUTHOR LIST
#
### END AUTHOR LIST
module Iof
  module Query
    VSA=Struct.new(:name,:internal_name,:devices,:uri,:suffix)
    Device=Struct.new(:name,:internal_name,:type,:image_file,:physical,:interfaces,:group,:uri,:suffix)
    Interface=Struct.new(:name,:internal_name,:vlans,:addresses,:connected,:internal_identifier,:uri,:suffix)
    Address=Struct.new(:type,:value,:network,:internal_name,:uri,:suffix)
    Network=Struct.new(:type,:value,:netmask_length,:internal_name,:uri,:suffix)
    Vlan=Struct.new(:internal_name,:id,:uri,:suffix)
    Group=Struct.new(:internal_name,:name,:uri,:suffix)

    class VisaRDF
      class << self
        def import(input,url_base="http://visa-project.de/topology/",format=:rdfxml,group_url_base="http://visa-project.de/backup/")
          reader=RDF::Reader.for(:rdfxml).new(input)

          graph=Hash.new()
          graph[:by_subject]=Hash.new()
          graph[:by_object]=Hash.new()
          graph[:root_nodes]=Array.new()
          graph[:connection_subjects]=Hash.new()
          graph[:interfaces]=Hash.new()

          reader.each_statement do |statement|
            s=statement.subject.to_s.sub(/#{url_base}/,'').intern
            s=s.to_s.sub(/#{group_url_base}/,'').intern if s.to_s.match(/#{group_url_base}/)
            graph[:root_nodes] << s if statement.subject.root?()
            p=statement.predicate.to_s.sub(/#{url_base}/,'').intern
            p=p.to_s.sub(/#{group_url_base}/,'').intern if p.to_s.match(/#{group_url_base}/)
            o=statement.object.to_s.sub(/#{url_base}/,'')
            o=o.to_s.sub(/#{group_url_base}/,'').intern if o.to_s.match(/#{group_url_base}/)
            o=o.intern unless statement.object.is_a?(RDF::Literal)
            if p==:connected
              graph[:connection_subjects][s]=o
            else
              graph[:by_subject][s]=Hash.new() unless graph[:by_subject].include?(s)
              graph[:by_object][o]=Hash.new() unless graph[:by_object].include?(o) || statement.object.is_a?(RDF::Literal)
              graph[:by_subject][s][p]=Array.new() unless graph[:by_subject][s].include?(p)
              graph[:by_object][o][p]=Array.new() unless !graph[:by_object].include?(o) || graph[:by_object][o].include?(p) || statement.object.is_a?(RDF::Literal)
              graph[:by_subject][s][p] << o
              graph[:by_object][o][p] << s unless statement.object.is_a?(RDF::Literal)
            end
          end
          graph[:root_nodes] = graph[:by_subject].keys - graph[:by_object].keys

          vsas=Array.new()
          graph[:root_nodes].each do |root|
            v=Iof::Query::VSA.new(root.to_s)
            vsas << v
            current=graph[:by_subject][root]
            v[:internal_name]=root.to_s
            if root.to_s.match(/_[a-z0-9A-Z]{32}$/) then
              v[:suffix]=root.to_s.sub(/.*_([a-z0-9A-Z]{32}$)/,'\1')
              v[:uri]=root.to_s.sub(/_[a-z0-9A-Z]{32}$/,'')
            else
              v[:suffix]=''
              v[:uri]=root.to_s
            end
            conversion_helper(current,v,graph)
          end
          return vsas
        end

        def import_from_csv(vsa,input)
          result={vsa.intern => VSA.new(vsa,"__vsa_#{vsa}__",Hash.new())}
          input.split("\n").each do |l|
            next if l.match(/^#/)
            next if l.match(/^$/)
            line=l.strip()
            sp_line=line.split(';')
            sp_line.each { |s| s.strip!() }
            device=nil
            if result[vsa.intern][:devices].include?(sp_line[0].intern) then
              device=result[vsa.intern][:devices][sp_line[0].intern]
            else
              device=Device.new(sp_line[0],sp_line[0],sp_line[6].intern,nil,true,Hash.new())
              device[:image_file]=sp_line[7] unless sp_line[7].nil?() || sp_line[7].empty?()
              result[vsa.intern][:devices][sp_line[0].intern]=device
            end
            interface=Interface.new("#{device[:name]}___#{sp_line[1]}","#{device[:name]}___#{sp_line[1]}",Array.new(),Array.new(),nil)
            result[vsa.intern][:devices][sp_line[0].intern][:interfaces][interface[:name].intern]=interface
            interface[:internal_identifier]=sp_line[1]
            interface[:connected]="#{sp_line[8]}___#{sp_line[9]}" unless sp_line[8].nil?() || sp_line[8].empty?() || sp_line[9].nil?() || sp_line[9].empty?()
            sp_line[3].split(',').each do |vlan|
              interface[:vlans] << Vlan.new("__vlan_#{vlan}__",vlan)
            end
            sp_line[2].split(',').each do |mac|
              interface[:addresses] << Address.new(:ethernet,mac,nil,"__mac_#{mac}__")
            end
            sp_line[4].split(',').each do |ipv4|
              if ipv4.match(/\//) then
                a=ipv4.sub(/\/.*/,'')
                l=ipv4.sub(/.*\//,'')
                n=Iof::Data.ipv4_network_part(a,l)
                network=Network.new(:ipv4,n,l,"__ipv4_network_#{n}/#{l}__")
                interface[:addresses] << Address.new(:ipv4,a,network,"__ipv4_address_#{ipv4}__")
              else
                interface[:addresses] << Address.new(:ipv4,ipv4,nil,"__ipv4_address_#{ipv4}__")
              end
            end
            sp_line[5].split(',').each do |ipv6|
              if ipv6.match(/\//) then
                a=ipv6.sub(/\/.*/,'')
                l=ipv6.sub(/.*\//,'')
                n=Iof::Data.ipv6_network_part(a,l)
                network=Network.new(:ipv6,n,l,"__ipv6_network_#{n}/#{l}__")
                interface[:addresses] << Address.new(:ipv6,a,network,"__ipv6_address_#{ipv6}__")
              else
                interface[:addresses] << Address.new(:ipv6,ipv6,nil,"__ipv6_address_#{ipv6}__")
              end
            end
          end
          return result
        end

        def export(vsas,file_name=nil,url_base="http://visa-project.de/topology/",format=:rdfxml,group_url_base="http://visa-project.de/backup/")
          base=url_base
          graph=nil
          if base.nil?() then
            base=""
            graph = RDF::Graph.new()
          else
            graph = RDF::Graph.new(base.sub(/\/$/,''))
          end
          vsas.values.each do |v|
            graph << create_statement(base, v[:internal_name], "internalName", v[:internal_name],true)
            graph << create_statement(base, v[:internal_name], "name", v[:name],true)
            v[:devices].values.each do |d|
              graph << create_statement(base, v[:internal_name], "device", d[:internal_name])
              graph << create_statement(base, d[:internal_name], "internalName", d[:internal_name],true)
              graph << create_statement(base, d[:internal_name], "name", d[:name],true)
              graph << create_statement(base, d[:internal_name], "type", d[:type].to_s,true)
              graph << create_statement(base, d[:internal_name], "physical", d[:physical].to_s,true)
              graph << create_statement(base, d[:internal_name], "imageFile", d[:image_file],true)
              unless d[:group].nil?()
                g=d[:group]
                graph << create_statement({:s => base , :p => group_url_base, :o => group_url_base}, d[:internal_name], "group", g[:internal_name])
                graph << create_statement(group_url_base, g[:internal_name], "name", g[:name].to_s,true)
              end
              d[:interfaces].values.each do |i|
                graph << create_statement(base, d[:internal_name], "interface", i[:internal_name])
                graph << create_statement(base, i[:internal_name], "internalName", i[:internal_name],true)
                graph << create_statement(base, i[:internal_name], "internalIdentifier", i[:internal_identifier],true)
                graph << create_statement(base, i[:internal_name], "name", i[:name],true)
                graph << create_statement(base, i[:internal_name], "connected", i[:connected]) unless i[:connected].nil?() || i[:connected]==""
                i[:vlans].each do |vlan|
                  graph << create_statement(base, i[:internal_name], "vlan", vlan[:internal_name])
                  graph << create_statement(base, vlan[:internal_name], "internalName", vlan[:internal_name],true)
                  graph << create_statement(base, vlan[:internal_name], "id", vlan[:id],true)
                end unless i[:vlans].nil?()
                i[:addresses].each do |a|
                  graph << create_statement(base, i[:internal_name], "address", a[:internal_name])
                  graph << create_statement(base, a[:internal_name], "type", a[:type].to_s,true)
                  graph << create_statement(base, a[:internal_name], "value", a[:value],true)
                  unless a[:network].nil?()
                    n=a[:network]
                    graph << create_statement(base, a[:internal_name], "network", n[:internal_name])
                    graph << create_statement(base, n[:internal_name], "type", n[:type].to_s,true)
                    graph << create_statement(base, n[:internal_name], "value", n[:value],true)
                    graph << create_statement(base, n[:internal_name], "netmaskLength", n[:netmask_length],true)
                  end
                end unless i[:addresses].nil?()
              end unless d[:interfaces].nil?()
            end unless v[:devices].nil?()
          end unless vsas.nil?()

          output=RDF::Writer.for(format).buffer() { |writer| writer << graph}
          File.open(file_name,'w') { |f| f.puts(output)} unless file_name.nil?() || file_name==""
          return {:graph => graph, :output => output}
        end

      private
        def conversion_helper(current,data,graph)
          current.each do |p,objects|
            objects.each do |o|
              next_klass=nil
              field=p
              multiple=false
              case p
              when :device
                field=:devices
                multiple=true
                next_klass=Iof::Query::Device
              when :interface
                field=:interfaces
                multiple=true
                next_klass=Iof::Query::Interface
              when :vlan
                field=:vlans
                multiple=true
                next_klass=Iof::Query::Vlan
              when :address
                field=:addresses
                multiple=true
                next_klass=Iof::Query::Address
              when :network
                next_klass=Iof::Query::Network
              when :group
                next_klass=Iof::Query::Group
              when :internalName
                field=:internal_name
              when :internalIdentifier
                field=:internal_identifier
              when :imageFile
                field=:image_file
              when :netmaskLength
                field=:netmask_length
              end
              if o.is_a?(String)
                if multiple
                  data[field]=Array.new() if data[field].nil?()
                  data[field] << o
                else
                  if field==:type
                    data[field]=o.intern
                  else
                    data[field]=o
                  end
                end
              else
                n=next_klass.new()
                if o.to_s.match(/_[a-z0-9A-Z]{32}$/) then
                  n[:suffix]=o.to_s.sub(/.*_([a-z0-9A-Z]{32}$)/,'\1')
                  n[:uri]=o.to_s.sub(/_[a-z0-9A-Z]{32}$/,'')
                else
                  n[:suffix]=''
                  n[:uri]=o.to_s
                end
                graph[:interfaces][o]=n if p==:interface
                if n.is_a?(Iof::Query::Interface) && graph[:connection_subjects].include?(o.to_s.intern)
                  fixed=graph[:connection_subjects][o.to_s.intern].to_s.sub(/(.*?)___(.*?)_([a-z0-9A-Z]{32}$)/,'\1_\3___\2').intern
                  n[:connected]=fixed
                end
                c=graph[:by_subject][o]
                n[:internal_name]=o.to_s.sub(/_[a-z0-9A-Z]{32}$/,'')
                if multiple
                  data[field]=Array.new() if data[field].nil?()
                  data[field] << n
                else
                  data[field]=n
                end
                conversion_helper(c, n, graph) unless c.nil?()
              end
            end
          end
        end

        def create_statement(base,s,p,o,datatype_property=false)
          if base.is_a?(Hash)
            s_base=base[:s]
            p_base=base[:p]
            o_base=base[:p]
          else
            s_base=base
            p_base=base
            o_base=base
          end
          subject=RDF::URI.new("#{s_base}#{s}")
          predicate=RDF::URI.new("#{p_base}#{p}")
          object=nil
          if datatype_property then
            object=RDF::Literal.new("#{o}")
          else
            object=RDF::URI.new("#{o_base}#{o}")
          end
          RDF::Statement.new(subject,predicate,object)
        end
      end
    end
  end
end
