# encoding: utf-8
# license: gpl3p 

### BEGIN LICENSE NOTICE
# This file is part of the interconnect asset ontology framework (IO)
# Copyright (C) 2010 - 2013
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE NOTICE

### BEGIN AUTHOR LIST
#
### END AUTHOR LIST
module Iof
  module Target
    class Item
      alias :generic_prepare :prepare

      def prepare(auth_handler)
        return generic_prepare(auth_handler) unless @device_vendor=='visa'
        unless @status_identify
          error "Could not prepare unidentified target #{description()}"
          @status_prepare=false
        else
          @auth_item=auth_handler[@auth_name.intern]
          vendor=nil
          vendor=@device_vendor.intern unless @device_vendor.nil?()
          type=nil
          type=@device_type.intern unless @device_type.nil?()
          connection=CONNECTION_SELECTOR.select_member([vendor,type])
          @connection_type=connection[0] unless connection.nil?()
          @connection_version=connection[1] unless connection.nil?()
          ctype=nil
          ctype=@connection_type.intern unless @connection_type.nil?()
          cversion=nil
          cversion=Integer(@connection_version) unless @connection_version.nil?()
          @commands=COMMAND_SELECTOR.select_member([vendor,type,ctype,cversion])
          @commands[:nova_create_images][:command]=@special[:image_script] if @commands.include?(:nova_create_images) && @special.include?(:image_script)
          @commands.each do |label,command|
            command[:command]="export LC_ALL='C'\nexport LANG='C'\nexport LANGUAGE='C'\n%CREDENTIALS%\n#{command[:command]}" unless label==:nova_create_images
            command[:command].gsub!(/%CREDENTIALS%/,@special[:credentials])
          end

          @connection_reconnect_per_command=RECONNECT_SELECTOR.select_member([vendor,type,ctype,cversion])
          @connection_class=Ghun::Remote::REMOTE_CONNECTION_SELECTOR.select_member([ctype,cversion])
          @parser_class=Iof::Parser::SELECTOR.select_member([vendor,type,ctype,cversion])
          @status_prepare=false if @connection_type.nil?()
          @status_prepare=false if @connection_class.nil?()
          @status_prepare=false if @parser_class.nil?()
          if @commands.nil?()
            error "No known commands for vendor '#{[vendor,type,ctype,cversion]}'"
            @status_prepare=false
          else
            @results=Hash.new()
            @commands.keys.each do |label|
              @results[label]=nil
            end
          end
          @status_prepare=true if @status_prepare.nil?()
        end
        return @status_prepare
      rescue => e
        error "Failed to prepare target #{description()}", e
        @status_prepare=false
        return @status_prepare
      end
    end
  end
end
