# encoding: utf-8
# license: gpl3p 

### BEGIN LICENSE NOTICE
# This file is part of the interconnect asset ontology framework (IO)
# Copyright (C) 2010 - 2013
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE NOTICE

### BEGIN AUTHOR LIST
#
### END AUTHOR LIST
module Iof
  module Target
    class TargetSourceVisaServer < TargetSource
      Iof::Target::Handler::SELECTOR.register_member(self,:visa_server)

      include Ghun::Base::Configuration
      def initialize(name)
        super(name)
        config.declare_dynamic_key(:"target.source.#{@name}.name",:string)
        config.declare_dynamic_key(:"target.source.#{@name}.host_names",:array)
        config.declare_dynamic_key(:"target.source.#{@name}.host_addresses_v4",:array)
        config.declare_dynamic_key(:"target.source.#{@name}.host_addresses_v6",:array)
        config.declare_dynamic_key(:"target.source.#{@name}.types",:array)
        config.declare_dynamic_key(:"target.source.#{@name}.auth",:array)
      end

      def load_targets(handler,mode,output)
        return unless enabled?()
        debug "#{self.class} loading targets in #{mode} mode"
        result=nil
        case mode
        when :execute
          result=load_from_config()
        when :read
          result=output.read(['target.source',@name])
        when :execute_or_read, :execute_and_store_or_read, :execute_and_store, :read_or_execute, :read_or_execute_and_store
          result=output.send(mode,['target.source',@name]) { load_from_config() }
        end

#        item=Item.new()
#        item.add_name(handler,vsa_name) unless vsa_name.nil?()
#        item.device_vendor='visa'
#        item.device_type='vsa'
#        item.auth_name='none'


        if result[:ipv4].size()!=result[:names].size() || result[:ipv6].size()!=result[:names].size() || result[:types].size()!=result[:names].size() || result[:auth].size()!=result[:names].size() then
          error "Array sizes for 'target.source.visa_server.*' configuration keys are inconsistent"
          raise InconsistentConfigurationError,"Array sizes for 'target.source.visa_server.*' configuration keys are inconsistent"
        else
          trace "Got data for #{result[:names].size()} VSA hosts"
        end

        (0..(result[:names].size()-1)).each do |i|

          item=Item.new()
          item.add_name(handler,result[:names][i]) unless result[:names][i].nil?()
          item.add_ipv4(handler,result[:ipv4][i]) unless result[:ipv4][i].nil?()
          item.add_ipv6(handler,result[:ipv6][i]) unless result[:ipv6][i].nil?()
          item.auth_name=result[:auth][i] unless result[:auth][i].nil?()
          item.device_vendor="visa"
          item.device_type=result[:types][i] unless result[:types][i].nil?()

          item.special[:vsa]=result[:vsa_name]
          item.special[:credentials]=result[:credentials]
          item.special[:image_script]=File.read(File.join(result[:script_dir],"create_images.sh"))
          item.special[:image_script].sub!(/%CREDENTIALS%/,result[:credentials])
          item.special[:image_script].sub!(/%HOST%/,result[:host])
          item.special[:image_script].sub!(/%USER%/,result[:user])
          item.special[:image_script].sub!(/%DIR%/,result[:dir])
          item.special[:image_script].sub!(/%KEY%/,result[:key])
        end

        trace "Loaded new VSA from configuration"
       end

      def load_from_config()
        result=Hash.new()
        result[:vsa_name]=config[:"target.source.visa_server.name"]
        result[:names]=config[:"target.source.visa_server.host_names"]
        result[:ipv4]=config[:"target.source.visa_server.host_addresses_v4"]
        result[:ipv6]=config[:"target.source.visa_server.host_addresses_v6"]
        result[:types]=config[:"target.source.visa_server.types"]
        result[:auth]=config[:"target.source.visa_server.auth"]
        result[:host]=config[:"visa.source.reverse_connection_host"]
        result[:dir]=config[:"visa.image_dir"]
        result[:user]=config[:"visa.source.reverse_connection_user"]
        result[:key]=File.read(config[:"visa.source.reverse_connection_key"])
        result[:script_dir]=config[:"visa.script_dir"]
        result[:credentials]="export OS_TENANT_NAME=#{config[:"visa.source.os_tenant"]}\nexport OS_USERNAME=#{config[:"visa.source.os_user"]}\nexport OS_PASSWORD=#{config[:"visa.source.os_password"]}\nexport OS_AUTH_URL=#{config[:"visa.source.os_url"]}\nexport SERVICE_ENDPOINT=#{config[:"visa.source.os_service_endpoint"]}\nexport SERVICE_TOKEN=#{config[:"visa.source.os_service_token"]}\n"
        return result
      end
    end
  end
end
